<?php

namespace App\Http\Controllers;

use App\Models\Currency;
use App\Models\Deposit;
use App\Models\Payment;
use App\Models\PaymentGateway;
use App\Models\Store;
use App\Models\User;
use App\PaymentMethods\PaymentMethod;
use App\PaymentMethods\Wallet;
use GuzzleHttp\Client;
use GuzzleHttp\Exception\GuzzleException;
use Illuminate\Auth\Access\AuthorizationException;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\URL;
use Illuminate\Support\Facades\Validator;
use Illuminate\Validation\ValidationException;

class PaymentController extends Controller
{
    function showDepositForm()
    {
        $currencies = Currency::query()->with(['gateways' => function ($q) {
            $q->where('payment_method', '!=', Wallet::class);
        }])->where('status', 1)->get();
        return view('user.deposit', compact('currencies'));
    }

    function deposit(Request $request)
    {
        $request->validate([
            'currency_id' => 'required',
            'amount' => 'required|numeric|max:99999',
            'gateway_id' => 'required',
        ]);
        /* @var Currency|null $currency */
        $currency = Currency::query()->where('id', $request->currency_id)->where('status', 1)->first();
        if (!$currency) throw ValidationException::withMessages(['currency_id' => __('Invalid Currency')]);
        $gateway = $currency->gateways()->where('id', $request->gateway_id)->first();
        if (!$gateway) throw ValidationException::withMessages(['currency_id' => __('Invalid Gateway')]);

        if ($request->amount == 0) throw ValidationException::withMessages(['amount' => __('Invalid Amount')]);
        if ($gateway->min != -1 && $request->amount < $gateway->min) throw ValidationException::withMessages(['amount' => __('Invalid Amount')]);
        if ($gateway->max != -1 && $request->amount > $gateway->max) throw ValidationException::withMessages(['amount' => __('Invalid Amount')]);

        /* @var User $user */
        $user = $request->user();
        $amount = $request->amount;
        $charge = (($gateway->percent_charge / 100) * $amount) + $gateway->fixed_charge;
        try {
            DB::beginTransaction();
            $deposit = $user->deposits()->create([
                'currency_id' => $currency->id,
                'payment_gateway_id' => $gateway->id,
                'amount' => $amount,
                'charge' => $charge
            ]);
            if (!$deposit) throw new \Exception(__('Unable to store deposit'));
            DB::commit();
            return redirect()->route('payment.redirect', ['type' => 'deposit', 'track' => encrypt($deposit->id)]);
        } catch (\Exception $exception) {
            DB::rollBack();
            throw ValidationException::withMessages(['amount' => $exception->getMessage()]);
        }
    }

    function storePaymentInit(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'api_key' => 'required|exists:stores,api_key',
            'custom' => 'nullable',
            'status_url' => 'required',
            'success_url' => 'required',
            'failed_url' => 'required',
            'amount' => 'required|numeric',
            'currency' => 'required',
            'title' => 'required',
            'description' => 'nullable'
        ]);
        if ($validator->fails()) {
            return response()->json(['error' => true, 'message' => 'Invalid Data'], 404);
//            return view('payment.failed', ['paymentData' => ['message' => __('Invalid Data'), 'failed_url' => $request->failed_url]]);
        }
        if ($request->amount <= 0) {
            return response()->json(['error' => true, 'message' => 'Invalid Amount'], 404);
//            return view('payment.failed', ['paymentData' => ['message' => __('Invalid Amount'), 'failed_url' => $request->failed_url]]);
        }
        /* @var Store $store */
        $store = Store::query()->where('api_key', $request->api_key)->first();
        if (!$store) {
            return response()->json(['error' => true, 'message' => 'Invalid API Key'], 404);
//            return view('payment.failed', ['paymentData' => ['message' => __('Invalid API Key'), 'failed_url' => $request->failed_url]]);
        }
        /* @var Currency $currency */
        $currency = Currency::query()->where('code', strtoupper($request->currency))->first();
        if (!$currency) {
            return response()->json(['error' => true, 'message' => 'Invalid Currency Code'], 404);
//            return view('payment.failed', ['paymentData' => ['message' => __('Invalid Currency Code'), 'failed_url' => $request->failed_url]]);
        }
        $gateways = $store->payment_gateways()->where('payment_gateways.currency_id', $currency->id)->get();
        if (!$gateways->count()) {
            return response()->json(['error' => true, 'message' => 'No payment processor selected by store owner'], 404);
//            return view('payment.failed', ['paymentData' => ['message' => __('No payment processor selected by store owner'), 'failed_url' => $request->failed_url]]);
        }
        /* @var Payment $payment */
        $payment = Payment::create([
            'user_id' => $store->user_id,
            'store_id' => $store->id,
            'currency_id' => $currency->id,
            'custom' => $request->custom,
            'status_url' => $request->status_url,
            'success_url' => $request->success_url,
            'failed_url' => $request->failed_url,
            'amount' => $request->amount,
            'title' => $request->title,
            'description' => $request->description
        ]);
        if (!$payment) {
            return response()->json(['error' => true, 'message' => 'Unable to create session'], 404);
        }
        return response()->json(['error' => false, 'payment_url' => route('payment.view', encrypt($payment->id))]);
    }

    function storePaymentVerify(Request $request)
    {
        $request->validate([
            'api_key' => 'required',
            'api_secret' => 'required',
            'payment_track' => 'required'
        ]);

        /* @var Store $store */
        $store = Store::query()->where('api_key', $request->api_key)->where('api_secret', $request->api_secret)->first();
        if (!$store) throw new AuthorizationException;
        $payment = $store->payments()->findOrFail(decrypt($request->payment_track));
        $payment->makeHidden(['user_id', 'store_id', 'currency_id', 'payment_gateway_id', 'status_url', 'success_url', 'failed_url']);
        return response()->json($payment);
    }

    function showPaymentPage(Request $request, $payment_id)
    {
        /* @var Payment $payment */
        $payment = Payment::query()->with(['store', 'currency', 'payment_gateway'])->findOrFail(decrypt($payment_id));
        if (!$payment) return abort(404);
        if ($payment->status != 'initialized') return abort(404);
        if (now()->subMinutes(30)->gt($payment->created_at)) return abort(404); // You must pay into 30 min
        if ($payment->payment_gateway) return redirect()->route('payment.redirect', ['type' => 'store_payment', 'track' => encrypt($payment->id)]);
        $gateways = $payment->store->payment_gateways()->where('payment_gateways.currency_id', $payment->currency_id)->get();
        return view('payment.store', compact('payment', 'gateways'));
    }

    function storePaymentProceed(Request $request, $payment_id)
    {
        $request->validate([
            'gateway_id' => 'required'
        ]);
        /* @var Payment $payment */
        $payment = Payment::query()->with(['store', 'currency', 'payment_gateway'])->findOrFail(decrypt($payment_id));
        if (!$payment) return abort(404);
        if ($payment->status != 'initialized') return abort(404);
        if (now()->subMinutes(30)->gt($payment->created_at)) return abort(404); // You must pay into 30 min
        if ($payment->payment_gateway) return redirect()->route('payment.redirect', ['type' => 'store_payment', 'track' => encrypt($payment->id)]);

        $gateway = $payment->store->payment_gateways()->where('payment_gateways.id', $request->gateway_id)->first();
        if (!$gateway) throw ValidationException::withMessages(['gateway_id' => 'Invalid Gateway Selection']);
        if ($gateway->currency_id != $payment->currency_id) throw ValidationException::withMessages(['gateway_id' => 'Invalid Gateway Selection']);

        try {
            $r = $payment->update([
                'payment_gateway_id' => $gateway->id
            ]);
            if (!$r) throw new \Exception('Unable to update payment');
        } catch (\Exception $exception) {
            return $payment->sendFailedIPNAndRedirectToFailed('Unable to update payment');
        }
        return redirect()->route('payment.redirect', ['type' => 'store_payment', 'track' => encrypt($payment->id)]);
    }

    function payment(Request $request, $type, $track)
    {
        if ($type == 'deposit') {
            /* @var Deposit $payment */
            $payment = Deposit::query()->with(['payment_gateway'])->findOrFail(decrypt($track));
            /* @var PaymentGateway $gateway */
            $gateway = $payment->payment_gateway;
        }
        if ($type == 'store_payment') {
            /* @var Payment $payment */
            $payment = Payment::query()->with(['payment_gateway'])->findOrFail(decrypt($track));
            /* @var PaymentGateway $gateway */
            $gateway = $payment->payment_gateway;
        }
        $class_name = $gateway->payment_method;
        /* @var PaymentMethod $processor */
        $processor = new $class_name;
        return $processor->process($gateway, $payment);
    }

    function ipnReceiver(Request $request, $type, $track)
    {
        if ($type == 'deposit') {
            /* @var Deposit $payment */
            $payment = Deposit::query()->with(['payment_gateway'])->findOrFail(decrypt($track));
            /* @var PaymentGateway $gateway */
            $gateway = $payment->payment_gateway;
        }
        if ($type == 'store_payment') {
            /* @var Payment $payment */
            $payment = Payment::query()->with(['payment_gateway'])->findOrFail(decrypt($track));
            /* @var PaymentGateway $gateway */
            $gateway = $payment->payment_gateway;
        }
        $class_name = $gateway->payment_method;
        /* @var PaymentMethod $processor */
        $processor = new $class_name;
        return $processor->ipn($request, $gateway, $payment);
    }

    function successPayment($type, $track)
    {
        if ($type == 'store_payment') {
            $payment = Payment::findOrFailed(decrypt($track));
            return redirect($payment->success_url);
        }
        return view('payment.success', compact('type', 'track'));
    }

    function failedPayment($type, $track)
    {
        if ($type == 'store_payment') {
            $payment = Payment::findOrFailed(decrypt($track));
            return redirect($payment->failed_url);
        }
        return view('payment.failed', compact('type', 'track'));
    }
}
