<?php

namespace App\Http\Controllers\Staff;

use App\Models\BlogPost;
use App\Http\Controllers\Controller;
use App\Models\Language;
use App\Models\Tag;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Collection;
use Illuminate\Http\Request;
use Illuminate\Support\Arr;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use Yajra\DataTables\DataTableAbstract;

class BlogPostController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {
        if ($request->ajax()) {
            $query = BlogPost::query();
            /* @var DataTableAbstract $table */
            $table = datatables()->of($query);

            $table->editColumn('status', function (BlogPost $blogPost) {
                if ($blogPost->status == 1) {
                    return '<span class="badge badge-success">Enabled</span>';
                } elseif ($blogPost->status == 2) {
                    return '<span class="badge badge-danger">Disabled</span>';
                } else {
                    return '<span class="badge badge-warning">Pending</span>';
                }
            });

            $table->addColumn('actions', function (BlogPost $blogPost) {
                return '<a href="' . route('staff.blog-post.edit', $blogPost->id) . '" class="btn btn-warning btn-edit nimmu-btn nimmu-btn-warning"><i class="fa fa-edit"></i></a><button class="btn btn-danger btn-delete nimmu-btn nimmu-btn-danger ml-1" data-id="' . $blogPost->id . '"><i class="fa fa-trash"></i></button>';
            });
            $table->rawColumns(['title', 'slug', 'status', 'actions']);
            return $table->make(true);
        }
        return view('staff.blog.index');
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $languages = Language::query()->where('status', 1)->get();
        $tags = Tag::all();
        return view('staff.blog.create', compact('languages', 'tags'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $request->validate([
            'title' => 'required|string|max:255',
            'slug' => 'required|alpha_dash|max:255|unique:blog_posts,slug',
            'feature_image' => 'required|string|max:255',
            'status' => 'required|numeric|in:0,1,2',
            'summery' => 'required',
            'details' => 'required',

        ], [
            'status.*' => 'Please select a valid status',
        ]);
        DB::beginTransaction();
        try {
            $blogPostParams = $request->only(['title', 'summery', 'details','slug', 'feature_image', 'status']);
            /* @var BlogPost|null $blogPost */
            $blogPostParams['created_by']=Auth::id();
            $blogPost = BlogPost::create($blogPostParams);
            if (is_array($request->tags)) {
                $blogPost->tags()->attach($request->tags);
            }

            if (is_array($request->lang)) {
                $translations = [];
                foreach ($request->lang as $code => $columns) {
                    foreach ($columns as $column => $content) {
                        $translations[] = [
                            'language' => $code,
                            'column_name' => $column,
                            'content' => $content
                        ];
                    }
                }
                $blogPost->translations()->createMany($translations);
            }
        } catch (\Exception $exception) {
            DB::rollBack();
            return redirect()->back()->withInput()->withErrors($exception->getMessage());
        }
        DB::commit();
        return redirect()->route('staff.blog-post.index')->withSuccess('Blog post added successfully');
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit(BlogPost $blogPost)
    {
        if (!$blogPost) return abort(404);

        /* @var Collection $translations */
        $translations = $blogPost->translations;
        $translations = $translations->groupBy('language')->map(function ($items) {
            return collect($items)->pluck('content', 'column_name');
        })->toArray();
        $tags = Tag::all();
        $blog_post_tag_ids = $blogPost->tags()->pluck('tags.id')->toArray();
        $languages = Language::query()->where('status', 1)->get();
        return view('staff.blog.edit', compact( 'blogPost','tags', 'blog_post_tag_ids', 'translations', 'languages'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, BlogPost $blogPost)
    {
        if (!$blogPost instanceof BlogPost) return abort(404);
        $request->validate([
            'title' => 'required|string|max:255',
            'slug' => 'required|alpha_dash|max:255|unique:blog_posts,slug,' . $blogPost->id,
            'feature_image' => 'required|string|max:255',
            'status' => 'required|numeric|in:0,1,2',
            'summery' => 'required',
            'details' => 'required',

        ], [
            'status.*' => 'Please select a valid status',
        ]);
        DB::beginTransaction();
        try {
            $blogPostParams = $request->only(['title', 'summery', 'details','slug', 'feature_image', 'status']);
            $blogPostParams['updated_by']=Auth::id();
            $blogPost->update($blogPostParams);
            $blogPost->tags()->detach();
            if (is_array($request->tags)) {
                $blogPost->tags()->attach($request->tags);
            }

            $blogPost->translations()->delete();
            if (is_array($request->lang)) {
                $translations = [];
                foreach ($request->lang as $code => $columns) {
                    foreach ($columns as $column => $content) {
                        $translations[] = [
                            'language' => $code,
                            'column_name' => $column,
                            'content' => $content
                        ];
                    }
                }
                $blogPost->translations()->createMany($translations);
            }
        } catch (\Exception $exception) {
            DB::rollBack();
            return redirect()->back()->withInput()->withErrors($exception->getMessage());
        }
        DB::commit();
        return redirect()->route('staff.blog-post.index')->withSuccess('Blog Post updated successfully');
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy(BlogPost $blogPost)
    {
        if (!$blogPost instanceof BlogPost) return abort(404);
        DB::beginTransaction();
        try {
            $blogPost->translations()->delete();
            $blogPost->tags()->detach();
            $r = $blogPost->delete();
            if (!$r) throw new \Exception('Unable to delete blog post');
        } catch (\Exception $exception) {
            DB::rollBack();
            return redirect()->back()->withInput()->withErrors($exception->getMessage());
        }
        DB::commit();
        return redirect()->route('staff.blog-post.index')->withSuccess('Blog Post deleted successfully');
    }
}
