<?php

namespace App\Http\Controllers\Staff;

use App\Models\Currency;
use App\Models\User;
use App\Models\Wallet;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Validation\ValidationException;
use Yajra\DataTables\DataTableAbstract;

class CurrencyController
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Contracts\Foundation\Application|\Illuminate\Contracts\View\Factory|\Illuminate\Http\JsonResponse|\Illuminate\View\View
     */
    public function index(Request $request)
    {
        if (Currency::where('status',1)->count()<=0) {
            Currency::generateDefaultCurrency();
        }
        if ($request->ajax()) {
            $query = Currency::query();
            /* @var DataTableAbstract $table */
            $table = datatables()->of($query);
            $table->editColumn('is_default', function (Currency $currency) {
                return $currency->is_default == 1 ? '<span class="badge badge-primary">Yes</span>' : '<span class="badge badge-warning">No</span>';
            });
            $table->editColumn('status', function (Currency $currency) {
                return $currency->status == 1 ? '<span class="badge badge-primary">Enabled</span>' : '<span class="badge badge-warning">Disabled</span>';
            });
            $table->addColumn('actions', function (Currency $currency) {
                return '<a class="btn btn-sm btn-warning mr-2" href="' . route('staff.setting.currency.edit', $currency->id) . '"><i class="fa fa-edit"></i></a> <button class="btn btn-danger btn-delete btn-sm" data-id="' . $currency->id . '"><i class="fa fa-trash"></i></button>';
            });
            $table->rawColumns(['is_default', 'status', 'actions']);
            return $table->make(true);
        }
        $default = Currency::getDefaultCurrency();
        return view('staff.setting.currency.index', compact('default'));
    }

    public function create()
    {
        $default = Currency::getDefaultCurrency();
        return view('staff.setting.currency.create', compact('default'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $request->validate([
            'name' => 'required|string|max:255',
            'code' => 'required|alpha_dash|max:3|unique:currencies,code',
            'symbol' => 'required|max:10',
            'thousand_separator' => 'required|max:10',
            'decimal_separator' => 'required|max:10',
            'rate' => 'required|numeric',
            'symbol_position' => 'required|in:' . implode(',', array_keys(config('currencies.formats'))),
            'precision' => 'required|numeric',
            'is_default' => 'required|in:0,1',
            'status' => 'required|in:0,1',
        ]);
        $old = Currency::query()->count();
        if (!$old) {
            if ($request->is_default != 1) {
                throw ValidationException::withMessages(['is_default' => 'You must have to set your first currency as default']);
            }
        }

        $params = $request->only(['name', 'code', 'symbol', 'symbol_position', 'precision', 'thousand_separator', 'decimal_separator', 'rate', 'is_default', 'status']);

        if ($request->status != 1) {
            $params['status'] = 0;
        }

        if ($request->is_default == 1) {
            $params['rate'] = 1;
            $params['status'] = 1;
        } else {
            $params['is_default'] = 0;
        }

        try {
            DB::beginTransaction();
            $currency = Currency::create($params);
            if (!$currency) throw new \Exception('Unable to create currency');
            if ($currency->is_default == 1 && $old) {
                $res = Currency::query()->where('id', '!=', $currency->id)->update([
                    'is_default' => 0
                ]);
                if (!$res) throw new \Exception('Unable to update all currency');
            }
            $users = User::all();
            $userNewWallet = [];
            foreach ($users as $user){
                $userNewWallet[] = [
                    'user_id'=>$user->id,
                    'currency_id'=>$currency->id,
                    'balance'=>0,
                ];
            }
            Wallet::insert($userNewWallet);
            DB::commit();
            return redirect()->route('staff.setting.currency.edit', $currency->id)->withSuccess('Currency created successfully');
        } catch (\Exception $exception) {
            DB::rollBack();
            return redirect()->back()->withError($exception->getMessage());
        }
    }

    public function refresh(Request $request)
    {
        $default = Currency::getDefaultCurrency();
        if (!$default instanceof Currency) return redirect()->back()->withErrors('Unable to update currency rates');
        $query = Currency::query()->where('id', '!=', $default->id);
        $count = (clone $query)->count();
        if (!$count) return redirect()->back()->withSuccess('Currency rates updated successfully');
        $currencies = (clone $query)->pluck('code')->implode(',');
        try {
            $response = file_get_contents(
                sprintf(
                    "http://api.currencylayer.com/live?access_key=%s&currencies=%s&format=1&source=%s",
                    config('services.currencylayer.access_key'),
                    $currencies,
                    $default->code
                )
            );
            $response = json_decode($response, true);

            if ($response['success'] && $response['quotes'] && is_array($response['quotes'])) {
                $currencies = (clone $query)->get();
                foreach ($currencies as $currency) {
                    $key = $default->code . $currency->code;
                    if (!isset($response['quotes'][$key])) continue;
                    $currency->update([
                        'rate' => $response['quotes'][$key]
                    ]);
                }
            } elseif (isset($response['error']) && is_array($response['error']) && isset($response['error']['info'])) {
                throw new \Exception($response['error']['info']);
            } elseif (isset($response['error'])) {
                throw new \Exception($response['error']);
            } else {
                throw new \Exception('Unable to update currency rates');
            }
        } catch (\Exception $exception) {
            return redirect()->back()->withError($exception->getMessage());
        }
        return redirect()->back()->withSuccess('Currency rates updated successfully');
    }

    public function edit(Currency $currency)
    {
        if (!$currency) return abort(404);
        $default = Currency::getDefaultCurrency();
        return view('staff.setting.currency.edit', compact('default', 'currency'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @param \App\Models\Currency $currency
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, Currency $currency)
    {
        if (!$currency) return abort(404);
        $request->validate([
            'name' => 'required|string|max:255',
            'code' => 'required|alpha_dash|max:3|unique:currencies,code,' . $currency->id,
            'symbol' => 'required|max:10',
            'thousand_separator' => 'required|max:10',
            'decimal_separator' => 'required|max:10',
            'rate' => 'required|numeric',
            'symbol_position' => 'required|in:' . implode(',', array_keys(config('currencies.formats'))),
            'precision' => 'required|numeric',
            'is_default' => 'required|in:0,1',
            'status' => 'required|in:0,1',
        ]);
        $default = Currency::getDefaultCurrency();

        $another = Currency::query()->where('id', '!=', $currency->id)->count();
        if ($default->id == $currency->id) {
            if ($request->is_default != 1) {
                throw ValidationException::withMessages(['is_default' => 'You must set this currency as default']);
            }
        }

        $params = $request->only(['name', 'code', 'symbol', 'symbol_position', 'precision', 'thousand_separator', 'decimal_separator', 'rate', 'is_default', 'status']);

        if ($request->status != 1) {
            $params['status'] = 0;
        }

        if ($request->is_default == 1) {
            $params['rate'] = 1;
            $params['status'] = 1;
        } else {
            $params['is_default'] = 0;
        }

        try {
            DB::beginTransaction();
            $res = $currency->update($params);
            if (!$res) throw new \Exception('Unable to update currency');
            if ($params['is_default'] == 1 && $another) {
                $res = Currency::query()->where('id', '!=', $currency->id)->update([
                    'is_default' => 0
                ]);
                if (!$res) throw new \Exception('Unable to update all currency');
            }
            DB::commit();
            return redirect()->back()->withSuccess('Currency updated successfully');
        } catch (\Exception $exception) {
            DB::rollBack();
            return redirect()->back()->withError($exception->getMessage());
        }
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param \App\Models\Currency $currency
     * @return \Illuminate\Http\Response
     */
    public function destroy(Currency $currency)
    {
        if (!$currency) return abort(404);
        $default = Currency::getDefaultCurrency();
        if ($default->id == $currency->id) return redirect()->back()->withError('You are not allowed to delete your default currency');
        try {
            $res = $currency->delete();
            if (!$res) throw new \Exception('Unable to delete currency');
            return redirect()->back()->withSuccess('Currency deleted successfully');
        } catch (\Exception $exception) {
            return redirect()->back()->withError($exception->getMessage());
        }
    }
}
