<?php

namespace App\Http\Controllers\Staff;

use App\Http\Controllers\Controller;
use App\Models\Faq;
use App\Models\Language;
use Illuminate\Database\Eloquent\Collection;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Str;
use Yajra\DataTables\DataTableAbstract;

class FaqController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {
        if ($request->ajax()) {
            $query = Faq::query();
            /* @var DataTableAbstract $table */
            $table = datatables()->of($query);

            $table->editColumn('question', function (Faq $faq) {
                return Str::limit($faq->question, 50, ' (...)');
            });

            $table->editColumn('answer', function (Faq $faq) {
                return Str::limit($faq->answer, 50, ' (...)');
            });

            $table->editColumn('status', function (Faq $faq) {
                if ($faq->status == 1) {
                    return '<span class="badge badge-success">Enabled</span>';
                } else {
                    return '<span class="badge badge-danger">Disabled</span>';
                }
            });

            $table->addColumn('action', function (Faq $faq) {
                return '<a class="btn btn-warning nimmu-btn nimmu-btn-warning btn-xs" href="' . route('staff.faq.edit', $faq->id) . '"><i
                                                            class="fa fa-edit"></i></a>
                                                    <button class="btn btn-danger btn-delete nimmu-btn nimmu-btn-danger btn-xs"
                                                            data-id="' . $faq->id . '"><i
                                                            class="fa fa-trash"></i></button>';
            });
            $table->rawColumns(['question', 'answer', 'status', 'action']);
            return $table->make(true);
        }
        return view('staff.faq.index');
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $languages = Language::query()->where('status', 1)->get();
        return view('staff.faq.create',compact('languages'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $request->validate([
            'question' => 'required|max:250',
            'answer' => 'required|string',
            'status' => 'required|numeric|in:0,1',
        ]);
        DB::beginTransaction();
        try {
            $p = $request->only(['question', 'answer', 'status']);
            /* @var Faq $faq */
            $faq = Faq::create($p);
            if (!$faq) throw new \Exception('Unable to create new faq');

            if (is_array($request->lang)) {
                $translations = [];
                foreach ($request->lang as $code => $columns) {
                    foreach ($columns as $column => $content) {
                        $translations[] = [
                            'language' => $code,
                            'column_name' => $column,
                            'content' => $content
                        ];
                    }
                }
                $faq->translations()->createMany($translations);
            }
        } catch (\Exception $exception) {
            DB::rollBack();
            return redirect()->back()->withErrors($exception->getMessage());
        }
        DB::commit();
        return redirect()->route('staff.faq.index')->withSuccess('New Faq added successfully');
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit(Faq $faq)
    {
        if (!$faq) return abort(404);
        /* @var Collection $translations */
        $translations = $faq->translations;
        $translations = $translations->groupBy('language')->map(function ($items) {
            return collect($items)->pluck('content', 'column_name')->toArray();
        })->toArray();
        $languages = Language::query()->where('status', 1)->get();
        return view('staff.faq.edit', compact('faq', 'translations', 'languages'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, Faq $faq)
    {
        if (!$faq instanceof Faq) return abort(404);
        $request->validate([
            'question' => 'required|max:250',
            'answer' => 'required|string',
            'status' => 'required|numeric|in:0,1',
        ]);
        DB::beginTransaction();
        try {
            $p = $request->only(['question', 'answer', 'status']);
            $r = $faq->update($p);
            if (!$r) throw new \Exception('Unable to update faq');
            $faq->translations()->delete();
            if (is_array($request->lang)) {
                $translations = [];
                foreach ($request->lang as $code => $columns) {
                    foreach ($columns as $column => $content) {
                        $translations[] = [
                            'language' => $code,
                            'column_name' => $column,
                            'content' => $content
                        ];
                    }
                }
                $faq->translations()->createMany($translations);
            }
        } catch (\Exception $exception) {
            DB::rollBack();
            return redirect()->back()->withErrors($exception->getMessage());
        }
        DB::commit();
        return redirect()->back()->withSuccess('Faq updated successfully');
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy(Faq $faq)
    {
        if (!$faq instanceof Faq) return abort(404);
        DB::beginTransaction();
        try {
            $faq->translations()->delete();
            $r = $faq->delete();
            if (!$r) throw new \Exception('Unable to delete FAQ');
        } catch (\Exception $exception) {
            DB::rollBack();
            return redirect()->back()->withInput()->withErrors($exception->getMessage());
        }
        DB::commit();
        return redirect()->route('staff.faq.index')->withSuccess('FAQ deleted successfully');
    }
}
