<?php

namespace App\Http\Controllers\Staff;

use App\Models\FaqQuestion;
use App\Models\FaqSection;
use App\Http\Controllers\Controller;
use App\Models\Language;
use Illuminate\Database\Eloquent\Collection;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Yajra\DataTables\DataTableAbstract;

class FaqQuestionController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @param Request $request
     * @return \Illuminate\Http\Response
     * @throws \Exception
     */
    public function index(Request $request)
    {
        if ($request->ajax()) {
            $query = FaqQuestion::query();
            /* @var DataTableAbstract $table */
            $table = datatables()->of($query);
            $table->addColumn('action', function (FaqQuestion $question) {
                return '<a class="btn btn-warning nimmu-btn nimmu-btn-warning" href="' . route('staff.faq.edit', $question->id) . '"><i
                                                            class="fa fa-edit"></i></a>
                                                    <button class="btn btn-danger btn-delete nimmu-btn nimmu-btn-danger"
                                                            data-id="' . $question->id . '"><i
                                                            class="fa fa-trash"></i></button>';
            });
            return $table->make(true);
        }
        return view('staff.faq.index');
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $languages = Language::query()->where('status', 1)->get();
        $sections = FaqSection::all();
        return view('staff.faq.create', compact('languages', 'sections'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $request->validate([
            'question' => 'required',
            'answer' => 'required|string',
            'section_id' => 'required|exists:faq_sections,id'
        ]);
        DB::beginTransaction();
        try {
            $p = $request->only(['question', 'answer', 'section_id']);
            /* @var FaqQuestion $question */
            $question = FaqQuestion::create($p);
            if (!$question) throw new \Exception('Unable to create faq');

            if (is_array($request->lang)) {
                $translations = [];
                foreach ($request->lang as $code => $columns) {
                    foreach ($columns as $column => $content) {
                        $translations[] = [
                            'language' => $code,
                            'column_name' => $column,
                            'content' => $content
                        ];
                    }
                }
                $question->translations()->createMany($translations);
            }
        } catch (\Exception $exception) {
            DB::rollBack();
            return redirect()->back()->withInput()->withErrors($exception->getMessage());
        }
        DB::commit();
        return redirect()->route('staff.faq.index')->withSuccess('Faq created successfully');
    }

    /**
     * Display the specified resource.
     *
     * @param \App\Models\FaqQuestion $faqQuestion
     * @return \Illuminate\Http\Response
     */
    public function show(FaqQuestion $faqQuestion)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param \App\Models\FaqQuestion $faq
     * @return \Illuminate\Contracts\Foundation\Application|\Illuminate\Contracts\View\Factory|\Illuminate\Http\Response|\Illuminate\View\View
     */
    public function edit(FaqQuestion $faq)
    {
        if (!$faq) return abort(404);
        /* @var Collection $translations */
        $translations = $faq->translations;
        $translations = $translations->groupBy('language')->map(function ($items) {
            return collect($items)->pluck('content', 'column_name')->toArray();
        })->toArray();
        $languages = Language::query()->where('status', 1)->get();
        $sections = FaqSection::all();
        return view('staff.faq.edit', compact('faq', 'sections', 'translations', 'languages'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @param \App\Models\FaqQuestion $faq
     * @return \Illuminate\Http\RedirectResponse
     */
    public function update(Request $request, FaqQuestion $faq)
    {
        if (!$faq instanceof FaqQuestion) return abort(404);
        $request->validate([
            'question' => 'required',
            'answer' => 'required|string',
            'section_id' => 'required|exists:faq_sections,id'
        ]);
        DB::beginTransaction();
        try {
            $p = $request->only(['question', 'answer']);

            $r = $faq->update($p);
            if (!$r) throw new \Exception('Unable to create faq');

            $faq->translations()->delete();
            if (is_array($request->lang)) {
                $translations = [];
                foreach ($request->lang as $code => $columns) {
                    foreach ($columns as $column => $content) {
                        $translations[] = [
                            'language' => $code,
                            'column_name' => $column,
                            'content' => $content
                        ];
                    }
                }
                $faq->translations()->createMany($translations);
            }
        } catch (\Exception $exception) {
            DB::rollBack();
            return redirect()->back()->withErrors($exception->getMessage());
        }
        DB::commit();
        return redirect()->back()->withSuccess('Faq updated successfully');
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param \App\Models\FaqQuestion $faq
     * @return \Illuminate\Http\Response
     */
    public function destroy(FaqQuestion $faq)
    {
        if (!$faq) return abort(404);

        DB::beginTransaction();
        try {
            $faq->translations()->delete();
            $res = $faq->delete();
            if (!$res) return redirect()->back()->withErrors('Unable to delete faq');
        } catch (\Exception $exception) {
            DB::rollBack();
            return redirect()->back()->withErrors($exception->getMessage());
        }

        DB::commit();
        return redirect()->back()->withSuccess('Faq deleted successfully');
    }
}
