<?php

namespace App\Http\Controllers\Staff;

use App\Models\FaqSection;
use App\Http\Controllers\Controller;
use App\Models\Language;
use App\Models\Tag;
use Illuminate\Database\Eloquent\Collection;
use Illuminate\Database\QueryException;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Yajra\DataTables\DataTableAbstract;

class FaqSectionController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @param Request $request
     * @return \Illuminate\Contracts\Foundation\Application|\Illuminate\Contracts\View\Factory|\Illuminate\Http\Response|\Illuminate\View\View
     */
    public function index(Request $request)
    {
        if ($request->ajax()) {
            $query = FaqSection::query();
            /* @var DataTableAbstract $table */
            $table = datatables()->of($query);
            $table->editColumn('logo', function (FaqSection $section) {
                return sprintf('<img src="%s" class="img-thumbnail" style="width: 100px;height: 100px;">', $section->logo);
            });
            $table->addColumn('action', function (FaqSection $section) {
                return '<a class="btn btn-warning nimmu-btn nimmu-btn-warning" href="' . route('staff.faq-section.edit', $section->id) . '"><i
                                                            class="fa fa-edit"></i></a>
                                                    <button class="btn btn-danger btn-delete nimmu-btn nimmu-btn-danger"
                                                            data-id="' . $section->id . '"><i
                                                            class="fa fa-trash"></i></button>';
            });
            $table->rawColumns(['logo', 'action']);
            return $table->make(true);
        }
        return view('staff.faq.section.index');
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Contracts\Foundation\Application|\Illuminate\Contracts\View\Factory|\Illuminate\Http\Response|\Illuminate\View\View
     */
    public function create()
    {
        $languages = Language::query()->where('status', 1)->get();
        return view('staff.faq.section.create', compact('languages'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $request->validate([
            'name' => 'required',
            'description' => 'nullable|string',
            'image' => 'required'
        ]);
        DB::beginTransaction();
        try {
            $p = $request->only(['name', 'description']);
            $p['logo'] = $request->image;
            /* @var FaqSection $section */
            $section = FaqSection::create($p);
            if (!$section) throw new \Exception('Unable to create faq section');

            if (is_array($request->lang)) {
                $translations = [];
                foreach ($request->lang as $code => $columns) {
                    foreach ($columns as $column => $content) {
                        $translations[] = [
                            'language' => $code,
                            'column_name' => $column,
                            'content' => $content
                        ];
                    }
                }
                $section->translations()->createMany($translations);
            }
        } catch (\Exception $exception) {
            DB::rollBack();
            return redirect()->back()->withErrors($exception->getMessage());
        }
        DB::commit();
        return redirect()->route('staff.faq-section.index')->withSuccess('Section created successfully');
    }

    /**
     * Display the specified resource.
     *
     * @param \App\Models\FaqSection $faqSection
     * @return \Illuminate\Http\Response
     */
    public function show(FaqSection $faqSection)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param \App\Models\FaqSection $faqSection
     * @return \Illuminate\Contracts\Foundation\Application|\Illuminate\Contracts\View\Factory|\Illuminate\Http\Response|\Illuminate\View\View
     */
    public function edit(FaqSection $faqSection)
    {
        if (!$faqSection) return abort(404);
        /* @var Collection $translations */
        $translations = $faqSection->translations;
        $translations = $translations->groupBy('language')->map(function ($items) {
            return collect($items)->pluck('content', 'column_name');
        })->toArray();
        $languages = Language::query()->where('status', 1)->get();
        return view('staff.faq.section.edit', compact('faqSection', 'translations', 'languages'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @param \App\Models\FaqSection $faqSection
     * @return \Illuminate\Http\RedirectResponse
     */
    public function update(Request $request, FaqSection $faqSection)
    {
        if (!$faqSection instanceof FaqSection) return abort(404);
        $request->validate([
            'name' => 'required',
            'description' => 'nullable|string',
            'image' => 'required'
        ]);
        DB::beginTransaction();
        try {
            $p = $request->only(['name', 'description']);
            $p['logo'] = $request->image;

            $r = $faqSection->update($p);
            if (!$r) throw new \Exception('Unable to create faq section');

            $faqSection->translations()->delete();
            if (is_array($request->lang)) {
                $translations = [];
                foreach ($request->lang as $code => $columns) {
                    foreach ($columns as $column => $content) {
                        $translations[] = [
                            'language' => $code,
                            'column_name' => $column,
                            'content' => $content
                        ];
                    }
                }
                $faqSection->translations()->createMany($translations);
            }
        } catch (\Exception $exception) {
            DB::rollBack();
            return redirect()->back()->withErrors($exception->getMessage());
        }
        DB::commit();
        return redirect()->back()->withSuccess('Section updated successfully');
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param \App\Models\FaqSection $faqSection
     * @return \Illuminate\Http\RedirectResponse
     */
    public function destroy(FaqSection $faqSection)
    {
        if (!$faqSection) return abort(404);

        DB::beginTransaction();
        try {
            $faqSection->translations()->delete();
            $res = $faqSection->delete();
            if (!$res) return redirect()->back()->withErrors('Unable to delete section');
        } catch (\Exception $exception) {
            DB::rollBack();
            return redirect()->back()->withErrors($exception->getMessage());
        }

        DB::commit();
        return redirect()->back()->withSuccess('Section deleted successfully');
    }
}
