<?php

namespace App\Http\Controllers\Staff;

use App\Http\Controllers\Controller;
use App\Models\KeyFeature;
use App\Models\Language;
use Illuminate\Database\Eloquent\Collection;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;

class KeyFeatureController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $keyFeatures = KeyFeature::all();
        return view('staff.setting.key_feature.index',compact('keyFeatures'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $languages = Language::query()->where('status', 1)->get();
        return view('staff.setting.key_feature.create',compact('languages'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $request->validate([
            'title' => 'required|string|max:255',
            'details' => 'required|string',
            'image' => 'required|string|max:255',
            'action' => 'nullable|string|max:255'
        ]);
        DB::beginTransaction();
        try {
            $params = $request->only(['title', 'details', 'image','action']);
            /* @var KeyFeature|null $keyFeature */

            $keyFeature = KeyFeature::create($params);

            if (is_array($request->lang)) {
                $translations = [];
                foreach ($request->lang as $code => $columns) {
                    foreach ($columns as $column => $content) {
                        $translations[] = [
                            'language' => $code,
                            'column_name' => $column,
                            'content' => $content
                        ];
                    }
                }
                $keyFeature->translations()->createMany($translations);
            }
        } catch (\Exception $exception) {
            DB::rollBack();
            return redirect()->back()->withInput()->withErrors($exception->getMessage());
        }
        DB::commit();
        return redirect()->route('staff.setting.key-features.index')->withSuccess('New Key Feature added successfully');
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit(KeyFeature $keyFeature)
    {
        if (!$keyFeature) return abort(404);

        /* @var Collection $translations */
        $translations = $keyFeature->translations;
        $translations = $translations->groupBy('language')->map(function ($items) {
            return collect($items)->pluck('content', 'column_name');
        })->toArray();
        $languages = Language::query()->where('status', 1)->get();
        return view('staff.setting.key_feature.edit', compact( 'keyFeature', 'translations', 'languages'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, KeyFeature $keyFeature)
    {
        if (!$keyFeature instanceof KeyFeature) return abort(404);
        $request->validate([
            'title' => 'required|string|max:255',
            'details' => 'required|string',
            'image' => 'required|string|max:255',
            'action' => 'nullable|string|max:255'
        ]);
        DB::beginTransaction();
        try {
            $params = $request->only(['title', 'details', 'image','action']);

            $keyFeature->update($params);

            $keyFeature->translations()->delete();
            if (is_array($request->lang)) {
                $translations = [];
                foreach ($request->lang as $code => $columns) {
                    foreach ($columns as $column => $content) {
                        $translations[] = [
                            'language' => $code,
                            'column_name' => $column,
                            'content' => $content
                        ];
                    }
                }
                $keyFeature->translations()->createMany($translations);
            }
        } catch (\Exception $exception) {
            DB::rollBack();
            return redirect()->back()->withInput()->withErrors($exception->getMessage());
        }
        DB::commit();
        return redirect()->route('staff.setting.key-features.index')->withSuccess('Key Feature updated successfully');
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy(KeyFeature $keyFeature)
    {
        if (!$keyFeature instanceof KeyFeature) return abort(404);
        DB::beginTransaction();
        try {
            $keyFeature->translations()->delete();
            $r = $keyFeature->delete();
            if (!$r) throw new \Exception('Unable to delete blog post');
        } catch (\Exception $exception) {
            DB::rollBack();
            return redirect()->back()->withInput()->withErrors($exception->getMessage());
        }
        DB::commit();
        return redirect()->route('staff.setting.key-features.index')->withSuccess('Key Feature deleted successfully');
    }
}
