<?php

namespace App\Http\Controllers\Staff;

use App\Models\Language;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Validation\ValidationException;
use Yajra\DataTables\DataTableAbstract;

class LanguageController
{
    /**
     * Display a listing of the resource.
     *
     * @param Request $request
     * @return \Illuminate\Contracts\Foundation\Application|\Illuminate\Contracts\View\Factory|\Illuminate\Http\Response|\Illuminate\View\View
     * @throws \Exception
     */
    public function index(Request $request)
    {
        if ($request->ajax()) {
            $query = Language::query();
            /* @var DataTableAbstract $table */
            $table = datatables()->of($query);
            $table->editColumn('is_default', function (Language $language) {
                return $language->is_default == 1 ? '<span class="badge badge-primary">Yes</span>' : '<span class="badge badge-warning">No</span>';
            });
            $table->editColumn('status', function (Language $language) {
                return $language->status == 1 ? '<span class="badge badge-primary">Enabled</span>' : '<span class="badge badge-warning">Disabled</span>';
            });
            $table->addColumn('actions', function (Language $language) {
                return '<button class="btn btn-warning btn-edit nimmu-btn nimmu-btn-warning"
                                                            data-id="' . $language->id . '"
                                                            data-name="' . $language->name . '"
                                                            data-code="' . $language->code . '"
                                                            data-is_default="' . $language->is_default . '"
                                                            data-status="' . $language->status . '"><i
                                                            class="fa fa-edit"></i></button>
                                                    <button class="btn btn-danger btn-delete nimmu-btn nimmu-btn-danger"
                                                            data-id="' . $language->id . '"><i
                                                            class="fa fa-trash"></i></button>';
            });
            $table->rawColumns(['is_default', 'status', 'actions']);
            return $table->make(true);
        }
        return view('staff.setting.languages');
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @return \Illuminate\Http\RedirectResponse
     */
    public function store(Request $request)
    {
        $request->validate([
            'name' => 'required|string|max:255',
            'code' => 'required|string|max:10|unique:languages,code',
            'status' => 'nullable|in:0,1',
            'is_default' => 'nullable|in:0,1'
        ]);
        $old = Language::query()->count();
        if (!$old) {
            if ($request->is_default != 1) {
                throw ValidationException::withMessages(['is_default' => 'You must have to set your first language as default']);
            }
        }
        $params = $request->only(['name', 'code', 'status', 'is_default']);
        if ($request->status != 1) {
            $params['status'] = 0;
        }
        try {
            DB::beginTransaction();
            if ($request->is_default == 1) {
                DB::table('languages')->update(['is_default' => 0]);
            }
            $language = Language::create($params);
            if (!$language) throw new \Exception('Unable to create new language');
            DB::commit();
            return redirect()->back()->withSuccess('Language created successfully');
        } catch (\Exception $exception) {
            DB::rollBack();
            return redirect()->back()->withError($exception->getMessage());
        }
    }

    /**
     * Update the specified resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @param \App\Models\Language $language
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, Language $language)
    {
        if (!$language) return abort(404);
        $request->validate([
            'name' => 'required|string|max:255',
            'code' => 'required|string|max:10|unique:languages,code,' . $language->id,
            'status' => 'nullable|in:0,1',
            'is_default' => 'nullable|in:0,1'
        ]);
        $default = Language::getDefaultLanguage();
        if ($default->id == $language->id) {
            if ($request->is_default != 1) {
                throw ValidationException::withMessages(['is_default' => 'You must set this language as default']);
            }
        }

        $params = $request->only(['name', 'code', 'status', 'is_default']);
        if ($request->status != 1) {
            $params['status'] = 0;
        }
        try {
            DB::beginTransaction();
            if (($request->is_default == 1) && ($default->id != $language->id)) {
                DB::table('languages')->update(['is_default' => 0]);
            }
            $res = $language->update($params);
            if (!$res) throw new \Exception('Unable to update language');
            DB::commit();
            return redirect()->back()->withSuccess('Language updated successfully');
        } catch (\Exception $exception) {
            DB::rollBack();
            return redirect()->back()->withError($exception->getMessage());
        }
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param \App\Models\Language $language
     * @return \Illuminate\Http\Response
     */
    public function destroy(Language $language)
    {
        if (!$language) return abort(404);
        $default = Language::getDefaultLanguage();
        if ($default->id == $language->id) return redirect()->back()->withError('You are not allowed to delete your default language');
        try {
            $res = $language->delete();
            if (!$res) throw new \Exception('Unable to delete language');
            return redirect()->back()->withSuccess('Language deleted successfully');
        } catch (\Exception $exception) {
            return redirect()->back()->withError($exception->getMessage());
        }
    }
}
