<?php

namespace App\Http\Controllers\Staff;

use App\Models\Language;
use App\Models\Menu;
use App\Models\MenuItem;
use App\Models\Translation;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Str;
use Illuminate\Validation\ValidationException;
use Yajra\DataTables\DataTableAbstract;

class MenuController
{
    /**
     * Display a listing of the resource.
     *
     * @param Request $request
     * @return \Illuminate\Contracts\Foundation\Application|\Illuminate\Contracts\View\Factory|\Illuminate\Http\JsonResponse|\Illuminate\View\View
     */
    public function index(Request $request)
    {
        if ($request->ajax()) {
            $query = Menu::query();
            /* @var DataTableAbstract $table */
            $table = datatables()->of($query);
            $table->editColumn('location', function (Menu $menu) {
                $menu_item = 'theme.settings.menu.locations.'.$menu->location;
                if (config()->has($menu_item)) {
                    return config($menu_item);
                }
                return null;
            });
            $table->addColumn('action', function (Menu $menu) {
                $actions = sprintf('<a href="%s" class="btn btn-sm btn-success mr-1">%s</a>', route('staff.setting.menu.show', $menu->id), __('Items'));
                $actions .= sprintf('<button class="btn btn-sm btn-warning btn-edit mr-1" data-id="%s" data-title="%s" data-location="%s">%s</button>', $menu->id, $menu->title, $menu->location, __('Edit'));
                $actions .= sprintf('<button class="btn btn-sm btn-danger btn-delete" data-id="%s">%s</button>', $menu->id, __('Delete'));
                return $actions;
            });
            return $table->make(true);
        }
        return view('staff.setting.menu.index');
    }

    /**
     * Show the form for creating a new resource.
     *
     * @param Request $request
     * @param Menu $menu
     * @return array|\Illuminate\Http\Response
     */

    public function orderMenuItems($allMenuItems){
        foreach ($allMenuItems as $item){
            $translation = $item->translations;
            $translation = $translation->groupBy('language')->map(function ($items) {
                return collect($items)->pluck('content', 'column_name');
            })->toArray();
            $item['translation'] = $translation;
            if (count($item->children)>0){
                $this->orderMenuItems($item->children);
            }
        }
        return $allMenuItems;
    }

    public function getItems(Request $request, Menu $menu)
    {
        $menuItems = Menu::where('id', '=', $menu->id)
            ->with(['parent_items.children' => function ($q) {
                $q->orderBy('order');
            }])
            ->first();
        $allItems = $menuItems->parent_items->sortBy('order');
        $itemsWithChild = $this->orderMenuItems($allItems)->toArray();
        return response(is_array($itemsWithChild)?$itemsWithChild:[]);
    }

    /**
     * Show the form for creating a new resource.
     *
     * @param Request $request
     * @param Menu $menu
     * @return array|\Illuminate\Http\Response
     */
    public function updateItems(Request $request, Menu $menu)
    {
        $request->validate([
            'data' => 'nullable|array'
        ]);

        DB::beginTransaction();
        try {
            MenuItem::where('menu_id',$menu->id)->delete();
            Translation::where('translatable_type','App\Models\MenuItem')->delete();
            $this->orderMenu($menu->id, $request->data, null);
        } catch (\Exception $exception) {
            DB::rollBack();
            throw ValidationException::withMessages(['data' => $exception->getMessage()]);
        }
        DB::commit();
    }

    private function orderMenu($menuID, array $menuItems, $parentId)
    {
        foreach ($menuItems as $index => $menuItem) {
            $item = MenuItem::create(['menu_id'=>$menuID, 'text'=> $menuItem['text'], 'url'=>$menuItem['url'], 'target'=>$menuItem['target'], 'type'=>$menuItem['type'], 'parent_id'=>$parentId, 'order'=>($index + 1)]);
            $item->translations()->delete();
            if (is_array($menuItem['translation'])) {
                $translations = [];
                foreach ($menuItem['translation'] as $code => $columns) {
                    foreach ($columns as $column => $content) {
                        $translations[] = [
                            'language' => $code,
                            'column_name' => $column,
                            'content' => $content
                        ];
                    }
                }
                $item->translations()->createMany($translations);
            }
            if (isset($menuItem['children'])) {
                $this->orderMenu($menuID, $menuItem['children'], $item->id);
            }
        }
    }


    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $request->validate([
            'title' => 'required|string|max:191',
            'location' => 'required|max:191'
        ]);
        try {
            DB::beginTransaction();
            $menu = Menu::create($request->only(['title', 'location']));
            if (!$menu instanceof Menu) throw new \Exception('Unable to save menu');
            DB::commit();
            return redirect()->back()->withSuccess('Menu added successfully');
        } catch (\Exception $exception) {
            DB::rollBack();
            throw ValidationException::withMessages(['title' => $exception->getMessage()]);
        }
    }

    /**
     * Display the specified resource.
     *
     * @param  \App\Models\Menu  $menu
     * @return \Illuminate\Contracts\Foundation\Application|\Illuminate\Contracts\View\Factory|\Illuminate\Http\Response|\Illuminate\View\View|void
     */
    public function show(Menu $menu)
    {
        if (!$menu) return abort(404);
        $cards = array_map(function ($class_name) {
            $item = [
                'id' => Str::slug($class_name::getLabel()),
                'name' => $class_name::getLabel(),
                'records' => $class_name::menurecords()
            ];
            return $item;
        }, config('ui.menuables'));
        $languages = Language::query()->where('status', 1)->get()->toArray();
        return view('staff.setting.menu.show', compact('cards', 'menu','languages'));
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  \App\Models\Menu  $menu
     * @return \Illuminate\Http\Response
     */
    public function edit(Menu $menu)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\Models\Menu  $menu
     * @return void
     */
    public function update(Request $request, Menu $menu)
    {
        if (!$menu) return abort(404);
        $request->validate([
            'title' => 'required|string|max:191',
            'location' => 'required|max:191'
        ]);
        try {
            DB::beginTransaction();
            $res = $menu->update($request->only(['title', 'location']));
            if (!$res) throw new \Exception('Unable to update menu');
            DB::commit();
            return redirect()->back()->withSuccess('Menu updated successfully');
        } catch (\Exception $exception) {
            DB::rollBack();
            throw ValidationException::withMessages(['title' => $exception->getMessage()]);
        }
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\Models\Menu  $menu
     * @return \Illuminate\Http\Response
     */
    public function destroy(Menu $menu)
    {
        if (!$menu) return abort(404);
        try {
            DB::beginTransaction();
            $res = $menu->delete();
            if (!$res) throw new \Exception('Unable to delete menu');
            DB::commit();
            return redirect()->back()->withSuccess('Menu deleted successfully');
        } catch (\Exception $exception) {
            DB::rollBack();
            throw ValidationException::withMessages(['title' => $exception->getMessage()]);
        }
    }
}
