<?php

namespace App\Http\Controllers\Staff;

use App\Http\Controllers\Controller;
use App\Models\Language;
use App\Models\OurService;
use Illuminate\Database\Eloquent\Collection;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;

class OurServiceController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $ourServices = OurService::all();
        return view('staff.setting.our_service.index',compact('ourServices'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $languages = Language::query()->where('status', 1)->get();
        return view('staff.setting.our_service.create',compact('languages'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $request->validate([
            'title' => 'required|string|max:255',
            'details' => 'required|string',
            'image' => 'required|string|max:255'
        ]);
        DB::beginTransaction();
        try {
            $params = $request->only(['title', 'details', 'image']);
            /* @var OurService|null $ourService */

            $ourService = OurService::create($params);

            if (is_array($request->lang)) {
                $translations = [];
                foreach ($request->lang as $code => $columns) {
                    foreach ($columns as $column => $content) {
                        $translations[] = [
                            'language' => $code,
                            'column_name' => $column,
                            'content' => $content
                        ];
                    }
                }
                $ourService->translations()->createMany($translations);
            }
        } catch (\Exception $exception) {
            DB::rollBack();
            return redirect()->back()->withInput()->withErrors($exception->getMessage());
        }
        DB::commit();
        return redirect()->route('staff.setting.our-services.index')->withSuccess('New Service added successfully');
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit(OurService $ourService)
    {
        if (!$ourService) return abort(404);

        /* @var Collection $translations */
        $translations = $ourService->translations;
        $translations = $translations->groupBy('language')->map(function ($items) {
            return collect($items)->pluck('content', 'column_name');
        })->toArray();
        $languages = Language::query()->where('status', 1)->get();
        return view('staff.setting.our_service.edit', compact( 'ourService', 'translations', 'languages'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, OurService $ourService)
    {
        if (!$ourService instanceof OurService) return abort(404);
        $request->validate([
            'title' => 'required|string|max:255',
            'details' => 'required|string',
            'image' => 'required|string|max:255'
        ]);
        DB::beginTransaction();
        try {
            $params = $request->only(['title', 'details', 'image']);

            $ourService->update($params);

            $ourService->translations()->delete();
            if (is_array($request->lang)) {
                $translations = [];
                foreach ($request->lang as $code => $columns) {
                    foreach ($columns as $column => $content) {
                        $translations[] = [
                            'language' => $code,
                            'column_name' => $column,
                            'content' => $content
                        ];
                    }
                }
                $ourService->translations()->createMany($translations);
            }
        } catch (\Exception $exception) {
            DB::rollBack();
            return redirect()->back()->withInput()->withErrors($exception->getMessage());
        }
        DB::commit();
        return redirect()->route('staff.setting.our-services.index')->withSuccess('Service info updated successfully');
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy(OurService $ourService)
    {
        if (!$ourService instanceof OurService) return abort(404);
        DB::beginTransaction();
        try {
            $ourService->translations()->delete();
            $r = $ourService->delete();
            if (!$r) throw new \Exception('Unable to delete this service');
        } catch (\Exception $exception) {
            DB::rollBack();
            return redirect()->back()->withInput()->withErrors($exception->getMessage());
        }
        DB::commit();
        return redirect()->route('staff.setting.our-services.index')->withSuccess('Service deleted successfully');
    }
}
