<?php

namespace App\Http\Controllers\Staff;

use App\Http\Controllers\Controller;
use App\Models\Language;
use App\Models\Page;
use Illuminate\Auth\Access\AuthorizationException;
use Illuminate\Database\Eloquent\Collection;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Yajra\DataTables\DataTableAbstract;
use Illuminate\Validation\Rule;

class PageController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @param Request $request
     * @return \Illuminate\Contracts\Foundation\Application|\Illuminate\Contracts\View\Factory|\Illuminate\Http\JsonResponse|\Illuminate\Http\Response|\Illuminate\View\View
     */
    public function index(Request $request)
    {
        if (!Page::count()) {
            Page::createBuiltin();
        }
        if ($request->ajax()) {
            $query = Page::query();
            /* @var DataTableAbstract $table */
            $table = datatables()->of($query);

            $table->editColumn('created_at', function (Page $page) {
                return optional($page->created_at)->format('M d, Y h:i a');
            });

            $table->editColumn('status', function (Page $page) {
                if ($page->status == 1) {
                    return '<span class="badge badge-success">' . __('Published') . '</span>';
                } else {
                    return '<span class="badge badge-warning">' . __('Draft') . '</span>';
                }
            });

            $table->addColumn('actions', function (Page $page) {
                $actions = sprintf('<a href="%s" class="btn btn-warning btn-edit nimmu-btn nimmu-btn-warning"><i class="fa fa-edit"></i></a>', route('staff.page.edit', $page->id));
                if ($page->type == 'other') {
                    $actions .= sprintf('<button class="btn btn-danger btn-delete nimmu-btn nimmu-btn-danger ml-1" data-id="%s"><i class="fa fa-trash"></i></button>', $page->id);
                }
                return $actions;
            });
            $table->rawColumns(['status', 'actions']);
            return $table->make(true);
        }
        return view('staff.page.index');
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Contracts\Foundation\Application|\Illuminate\Contracts\View\Factory|\Illuminate\Http\Response|\Illuminate\View\View
     */
    public function create()
    {
        $languages = Language::query()->where('status', 1)->get();
        return view('staff.page.create', compact('languages'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @return \Illuminate\Http\RedirectResponse|\Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $request->validate([
            'title' => 'bail|required|string|max:191',
            'slug' => 'bail|required|slug|max:191|unique:pages,slug',
            'status' => 'bail|required|in:0,1',
            'type' => [
                'required',
                Rule::in(array_keys(config('theme.settings.page_templates'))),
            ]
        ]);

        DB::beginTransaction();
        try {
            $pageParams = $request->only(['title', 'slug', 'type','status', 'summery', 'content']);
            /* @var Page|null $page */
            $page = Page::create($pageParams);
            if (is_array($request->meta)) {
                $metas = [];
                foreach ($request->meta as $name => $content) {
                    $metas[] = [
                        'name' => $name,
                        'content' => $content
                    ];
                }
                $page->metas()->createMany($metas);
            }
            if (is_array($request->lang)) {
                $translations = [];
                foreach ($request->lang as $code => $columns) {
                    foreach ($columns as $column => $content) {
                        $translations[] = [
                            'language' => $code,
                            'column_name' => $column,
                            'content' => $content
                        ];
                    }
                }
                $page->translations()->createMany($translations);
            }
        } catch (\Exception $exception) {
            DB::rollBack();
            return redirect()->back()->withInput()->withErrors($exception->getMessage());
        }
        DB::commit();
        return redirect()->route('staff.page.index', $page->id)->withSuccess(__('Page added successfully'));
    }

    /**
     * Display the specified resource.
     *
     * @param \App\Models\Page $page
     * @return \Illuminate\Http\Response
     */
    public function show(Page $page)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param \App\Models\Page $page
     * @return \Illuminate\Contracts\Foundation\Application|\Illuminate\Contracts\View\Factory|\Illuminate\Http\Response|\Illuminate\View\View
     */
    public function edit(Page $page)
    {
        if (!$page) return abort(404);
        $metas = $page->metas()->pluck('content', 'name');
        /* @var Collection $translations */
        $translations = $page->translations;
        $translations = $translations->groupBy('language')->map(function ($items) {
            return collect($items)->pluck('content', 'column_name');
        })->toArray();
        $languages = Language::query()->where('status', 1)->get();
        return view('staff.page.edit', compact('page', 'metas', 'translations', 'languages'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @param \App\Models\Page $page
     * @return \Illuminate\Http\RedirectResponse|\Illuminate\Http\Response|void
     */
    public function update(Request $request, Page $page)
    {
        if (!$page) return abort(404);
        $r = [
            'title' => 'bail|required|string|max:191',
            'slug' => 'bail|required|slug|max:191|unique:pages,slug,' . $page->id,
            'status' => 'bail|required|in:0,1',
            'type' => [
                'required',
                Rule::in(array_keys(config('theme.settings.page_templates'))),
            ]
        ];

        $request->validate($r);

        DB::beginTransaction();
        try {
            $pageParams = $request->only(['title', 'slug', 'content', 'summery', 'type', 'status']);
            $res = $page->update($pageParams);
            if (!$res) throw new \Exception(__('Unable to update page'));

            $page->metas()->delete();
            if (is_array($request->meta)) {
                $metas = [];
                foreach ($request->meta as $name => $content) {
                    $metas[] = [
                        'name' => $name,
                        'content' => $content
                    ];
                }
                $page->metas()->createMany($metas);
            }

            $page->translations()->delete();
            if (is_array($request->lang)) {
                $translations = [];
                foreach ($request->lang as $code => $columns) {
                    foreach ($columns as $column => $content) {
                        $translations[] = [
                            'language' => $code,
                            'column_name' => $column,
                            'content' => $content
                        ];
                    }
                }
                $page->translations()->createMany($translations);
            }
        } catch (\Exception $exception) {
            DB::rollBack();
            return redirect()->back()->withInput()->withErrors($exception->getMessage());
        }
        DB::commit();
        return redirect()->back()->withSuccess(__('Page updated successfully'));
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param \App\Models\Page $page
     * @return \Illuminate\Http\Response
     */
    public function destroy(Page $page)
    {
        if (!$page instanceof Page) return abort(404);
        if ($page->type != 'other') throw new AuthorizationException;
        DB::beginTransaction();
        try {
            $page->translations()->delete();
            $page->metas()->delete();
            $r = $page->delete();
            if (!$r) throw new \Exception(__('Unable to delete page'));
        } catch (\Exception $exception) {
            DB::rollBack();
            return redirect()->back()->withInput()->withErrors($exception->getMessage());
        }
        DB::commit();
        return redirect()->route('staff.page.index')->withSuccess(__('Page deleted successfully'));
    }
}
