<?php

namespace App\Http\Controllers\Staff;

use App\Http\Controllers\Controller;
use App\Models\Currency;
use App\Models\PaymentGateway;
use Illuminate\Http\Request;
use Illuminate\Validation\ValidationException;
use Yajra\DataTables\DataTableAbstract;

class PaymentGatewayController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {
        if ($request->ajax()) {
            $query = PaymentGateway::query();
            /* @var DataTableAbstract $table */
            $table = datatables()->of($query);
            $table->editColumn('currency_id', function (PaymentGateway $paymentGateway) {
                if ($paymentGateway->currency instanceof Currency) return ($paymentGateway->currency->code);
            });
            $table->editColumn('payment_method', function (PaymentGateway $paymentGateway) {
                $invervePaymenMethods = array_flip(config('iwallet.payment_methods') ?? []);
                return $invervePaymenMethods[$paymentGateway->payment_method] ?? $paymentGateway->payment_method;
            });
            $table->editColumn('min', function (PaymentGateway $paymentGateway) {
                return $paymentGateway->min == -1 ? '<span class="badge badge-info">no limit</span>' : '<span class="badge badge-info">' . $paymentGateway->min . ' ' . $paymentGateway->currency->code . '</span>';
            });
            $table->editColumn('max', function (PaymentGateway $paymentGateway) {
                return $paymentGateway->max == -1 ? '<span class="badge badge-info">no limit</span>' : '<span class="badge badge-info">' . $paymentGateway->max . ' ' . $paymentGateway->currency->code . '</span>';
            });
            $table->addColumn('charge', function (PaymentGateway $paymentGateway) {
                return sprintf('<span class="badge badge-info">%s%% + %s %s</span>', $paymentGateway->percent_charge, $paymentGateway->fixed_charge, $paymentGateway->currency->code);
            });
            $table->editColumn('status', function (PaymentGateway $paymentGateway) {
                return $paymentGateway->status == 1 ? '<span class="badge badge-primary">Enabled</span>' : '<span class="badge badge-warning">Disabled</span>';
            });
            $table->addColumn('actions', function (PaymentGateway $paymentGateway) {
                return '<a class="btn btn-sm btn-warning mr-2" href="' . route('staff.setting.payment-gateway.edit', $paymentGateway->id) . '"><i class="fa fa-edit"></i></a> <button class="btn btn-danger btn-delete btn-sm" data-id="' . $paymentGateway->id . '"><i class="fa fa-trash"></i></button>';
            });
            $table->rawColumns(['payment_method', 'min', 'max', 'charge', 'currency_id', 'status', 'actions']);
            return $table->make(true);
        }
        return view('staff.setting.payment_gateway.index');
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $paymentMethods = [];
        $paymentMethodSettings = config('iwallet.payment_methods') ?? [];
        $currencies = Currency::where('status', 1)->pluck('code')->all();
        foreach ($paymentMethodSettings as $paymentMethodName => $paymentMethodClass) {
            if (class_exists($paymentMethodClass)) {
                try {
                    $paymentMethod = new $paymentMethodClass;
                    if (in_array('*', $paymentMethod->supported_currencies)) {
                        $supportedCurrencies = array_map('strtoupper', $currencies);
                    } else {
                        $supportedCurrencies = array_intersect(array_map('strtoupper', $currencies), array_map('strtoupper', $paymentMethod->supported_currencies));
                    }
                    $paymentMethodInfo = ['name' => $paymentMethodName, 'calssName' => $paymentMethodClass, 'supported_currencies' => $supportedCurrencies, 'credentials' => $paymentMethod->credentials];
                    array_push($paymentMethods, $paymentMethodInfo);
                } catch (\Exception $e) {

                }
            }
        }
        return view('staff.setting.payment_gateway.create', compact('paymentMethods'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $request->validate([
            'name' => 'required|string|max:191|unique:payment_gateways,name',
            'image' => 'nullable|string|max:191',
            'min' => 'nullable|numeric',
            'max' => 'nullable|numeric',
            'percent_charge' => 'required|numeric',
            'fixed_charge' => 'required|numeric',
            'payment_method' => 'required|string|max:191',
            'supported_currency' => 'required|string|max:191',
            'credentials' => 'nullable|array',
            'status' => 'required|in:0,1',
        ]);
        if (!class_exists($request->payment_method)) {
            throw ValidationException::withMessages(['payment_method' => 'Invalid Payment Method']);
        }
        $supportedCount = Currency::where([['code', $request->supported_currency], ['status', 1]])->first();
        if (is_null($supportedCount)) {
            throw ValidationException::withMessages(['supported_currency' => 'This currency is not available on the currency list']);
        }
        $params = $request->only(['name', 'image', 'min', 'max', 'percent_charge', 'fixed_charge' ,'payment_method', 'credentials', 'status']);
        $params['currency_id'] = $supportedCount->id;
        if ($request->status != 1) {
            $params['status'] = 0;
        }
        if (!$request->credentials) {
            $params['credentials'] = [];
        }
        $paymentGateway = PaymentGateway::create($params);
        if (!$paymentGateway) throw new \Exception('Unable to create payment gateway');
        return redirect()->route('staff.setting.payment-gateway.index')->withSuccess('New Payment Gateway Added Successfully');
    }

    /**
     * Display the specified resource.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function edit(PaymentGateway $paymentGateway)
    {
        if (!$paymentGateway) return abort(404);
        $paymentMethods = [];
        $selectedMethod = [];
        $paymentMethodSettings = config('iwallet.payment_methods') ?? [];
        $currencies = Currency::where('status', 1)->pluck('code')->all();
        foreach ($paymentMethodSettings as $paymentMethodName => $paymentMethodClass) {
            if (class_exists($paymentMethodClass)) {
                try {
                    $paymentMethod = new $paymentMethodClass;
                    if (in_array('*', $paymentMethod->supported_currencies)) {
                        $supportedCurrencies = array_map('strtoupper', $currencies);
                    } else{
                        $supportedCurrencies = array_intersect(array_map('strtoupper', $currencies), array_map('strtoupper', $paymentMethod->supported_currencies));
                    }
                    $paymentMethodInfo = ['name' => $paymentMethodName, 'calssName' => $paymentMethodClass, 'supported_currencies' => $supportedCurrencies, 'credentials' => $paymentMethod->credentials];
                    array_push($paymentMethods, $paymentMethodInfo);
                    if ($paymentMethodClass == $paymentGateway->payment_method) {
                        $selectedMethod = $paymentMethodInfo;
                    }
                } catch (\Exception $e) {

                }
            }
        }
        return view('staff.setting.payment_gateway.edit', compact('paymentGateway', 'paymentMethods', 'selectedMethod'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, PaymentGateway $paymentGateway)
    {
        if (!$paymentGateway) return abort(404);
        $request->validate([
            'name' => 'required|string|max:191|unique:payment_gateways,name,' . $paymentGateway->id,
            'image' => 'nullable|string|max:191',
            'min' => 'nullable|numeric',
            'max' => 'nullable|numeric',
            'percent_charge' => 'required|numeric',
            'fixed_charge' => 'required|numeric',
            'payment_method' => 'required|string|max:191',
            'supported_currency' => 'required|string|max:191',
            'credentials' => 'required|array',
            'status' => 'required|in:0,1',
        ]);
        if (!class_exists($request->payment_method)) {
            throw ValidationException::withMessages(['payment_method' => 'Invalid Payment Method']);
        }
        $supportedCount = Currency::where([['code', $request->supported_currency], ['status', 1]])->first();
        if (is_null($supportedCount)) {
            throw ValidationException::withMessages(['supported_currency' => 'This currency is not available on the currency list']);
        }
        $params = $request->only(['name', 'image', 'min', 'max', 'percent_charge', 'fixed_charge', 'payment_method', 'credentials', 'status']);
        $params['currency_id'] = $supportedCount->id;
        if ($request->status != 1) {
            $params['status'] = 0;
        }
        $res = $paymentGateway->update($params);
        if (!$res) throw new \Exception('Unable to update payment gateway');
        return redirect()->back()->withSuccess('Payment Gateway updated successfully');
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function destroy(PaymentGateway $paymentGateway)
    {
        if (!$paymentGateway) return abort(404);
        $res = $paymentGateway->delete();
        if (!$res) throw new \Exception('Unable to delete payment gateway');
        return redirect()->back()->withSuccess('Payment Gateway deleted successfully');
    }
}
