<?php

namespace App\Http\Controllers\Staff;

use App\Models\Currency;
use App\Events\TransactionAdded;
use App\Http\Controllers\Controller;
use App\Models\Transaction;
use App\Models\User;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Validation\ValidationException;
use Yajra\DataTables\DataTableAbstract;

class TransactionController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @param Request $request
     * @return \Illuminate\Contracts\Foundation\Application|\Illuminate\Contracts\View\Factory|\Illuminate\Http\JsonResponse|\Illuminate\View\View
     */
    public function index(Request $request)
    {
        if ($request->ajax()) {
            $query = Transaction::query()->with(['user']);
            if (in_array($request->user_type, ['user'])) {
                $user_type = User::class;
                $query->where('user_type', $user_type);
            }
            if (is_numeric($request->user_id)) {
                $query->where('user_id', $request->user_id);
            }
            if (in_array($request->type, ['+', '-'])) {
                $query->where('type', $request->type);
            }
            if ($request->start_date && ($start_date = Carbon::createFromFormat('d-m-Y', $request->start_date))) {
                $query->whereDate('created_at', '>=', $start_date);
            }
            if ($request->end_date && ($end_date = Carbon::createFromFormat('d-m-Y', $request->end_date))) {
                $query->whereDate('created_at', '<=', $end_date);
            }
            /* @var DataTableAbstract $table */
            $table = datatables()->of($query);
            $table->editColumn('created_at', function (Transaction $transaction) {
                return $transaction->created_at->format('M d, Y h:i a');
            });
            $table->editColumn('user.username', function (Transaction $transaction) {
                if ($transaction->user instanceof User) return ('<a href="' . route('staff.user.edit', $transaction->user->id) . '">' .$transaction->user->username . ' (Customer)'. '</a>');
            });
            $table->editColumn('amount', function (Transaction $transaction) {
                $currency = Currency::getDefaultCurrency();
                return sprintf('%s %s %s', $transaction->type, $transaction->amount, $currency->code);
            });
            $table->rawColumns(['user.username', 'amount', 'created_at']);
            return $table->make(true);
        }
        return view('staff.transaction.index');
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Contracts\Foundation\Application|\Illuminate\Contracts\View\Factory|\Illuminate\Http\Response|\Illuminate\View\View
     */
    public function create()
    {
        $currency = Currency::getDefaultCurrency();
        $customers = User::query()->where('status', 1)->get();
        return view('staff.transaction.create', compact('currency', 'customers'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @return array|\Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $t = 'users';
        $request->validate([
            'user_type' => 'required|in:user',
            'user_id' => 'required|exists:' . $t . ',id',
            'title' => 'required|string|max:191',
            'type' => 'required|in:+,-',
            'amount' => 'required|numeric|min:0'
        ]);
        /* @var User $user */
        $user = User::find($request->user_id);
        if ($request->type == '-' && $user->balance < $request->amount) throw ValidationException::withMessages(['amount' => 'Not Enough Balance']);

        try {
            DB::beginTransaction();
            if ($request->type == '+') {
                $r = $user->update([
                    'balance' => ($user->balance + $request->amount)
                ]);
                if (!$r) throw new \Exception('Unable to update balance');
                /* @var Transaction $transaction */
                $transaction = Transaction::create([
                    'track' => Transaction::generateTrack(),
                    'title' => $request->title,
                    'user_type' => get_class($user),
                    'user_id' => $user->id,
                    'type' => $request->type,
                    'amount' => $request->amount,
                    'matter' => 'system'
                ]);
            } else {
                $r = $user->update([
                    'balance' => ($user->balance - $request->amount)
                ]);
                if (!$r) throw new \Exception('Unable to update balance');
                /* @var Transaction $transaction */
                $transaction = Transaction::create([
                    'track' => Transaction::generateTrack(),
                    'title' => $request->title,
                    'user_type' => get_class($user),
                    'user_id' => $user->id,
                    'type' => $request->type,
                    'amount' => $request->amount,
                    'matter' => 'system'
                ]);
            }
            if (!$transaction) throw new \Exception('Unable to add transaction');
            DB::commit();
        } catch (\Exception $exception) {
            DB::rollBack();
            throw ValidationException::withMessages(['title' => $exception->getMessage()]);
        }
        event(new TransactionAdded($transaction));
        return redirect()->route('staff.transaction.index')->withSuccess('Transaction added successfully');
    }

    /**
     * Display the specified resource.
     *
     * @param \App\Models\Transaction $transaction
     * @return \Illuminate\Http\Response
     */
    public function show(Transaction $transaction)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param \App\Models\Transaction $transaction
     * @return \Illuminate\Http\Response
     */
    public function edit(Transaction $transaction)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @param \App\Models\Transaction $transaction
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, Transaction $transaction)
    {
        //
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param \App\Models\Transaction $transaction
     * @return \Illuminate\Http\Response
     */
    public function destroy(Transaction $transaction)
    {
        //
    }
}
