<?php

namespace App\Http\Controllers\Staff;

use App\Events\TransactionAdded;
use App\Events\UserRegistered;
use App\Models\Currency;
use App\Http\Controllers\Controller;
use App\Models\Role;
use App\Models\Staff;
use App\Models\Transaction;
use App\Models\User;
use App\Models\Wallet;
use App\Notifications\NotifyStaff;
use Carbon\Carbon;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Notification;
use Illuminate\Validation\Rule;
use Illuminate\Validation\ValidationException;
use Yajra\DataTables\DataTableAbstract;

class UserController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @param Request $request
     * @return \Illuminate\Contracts\Foundation\Application|\Illuminate\Contracts\View\Factory|\Illuminate\Http\JsonResponse|\Illuminate\Http\Response|\Illuminate\View\View
     * @throws \Exception
     */
    public function index(Request $request)
    {
        if ($request->ajax()) {
            $query = User::query();
            /* @var DataTableAbstract $table */
            $table = datatables()->of($query);
            $table->editColumn('name', function (User $user) {
                return ('<div class="d-flex">
                            <div class="image mr-2">
                                <img class="img-thumbnail" style="width: 50px;height: 50px;"
                                     src="' . $user->avatar . '"
                                     alt="' . $user->username . '">
                            </div>
                            <div class="meta">
                                <p>
                                    <a href="' . route('staff.user.edit', $user->id) . '">' . $user->first_name .' ' .$user->last_name . '</a>
                                </p>
                            </div>
                        </div>');
            });
            $table->addColumn('action', function (User $user) {
                return '<a href="' . route('staff.user-wallet', $user->id) . '" class="btn btn-success mr-1" data-toggle="tooltip" data-placement="top" title="Wallet"><i class="fa fa-wallet"></i></a><a href="' . route('staff.user.edit', $user->id) . '" class="btn btn-warning btn-edit mr-1"><i class="fa fa-edit"></i></a><button class="btn btn-danger btn-delete" data-id="' . $user->id . '"><i class="fa fa-trash"></i></button>';
            });
            $table->editColumn('status', function (User $user) {
                return $user->status == 1 ? '<span class="badge badge-success">Activated</span>' : '<span class="badge badge-danger">Deactivated</span>';
            });
            $table->filterColumn('name', function ($query, $keyword) {
                /* @var Builder $query */
                $query->where(function ($q) use ($keyword) {
                    /* @var Builder $q */
                    $q->where('first_name', 'LIKE', "%$keyword%")->orWhere('last_name', 'LIKE', "%$keyword%");
                });
            });
            $table->rawColumns(['name', 'status', 'action']);
            return $table->make(true);
        }
        return view('staff.user.index');
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Contracts\Foundation\Application|\Illuminate\Contracts\View\Factory|\Illuminate\Http\Response|\Illuminate\View\View
     */
    public function create()
    {
        $roles = Role::where('for','user')->get();
        return view('staff.user.create', compact('roles'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $request->validate([
            'first_name' => 'required|string|max:191',
            'last_name' => 'required|string|max:191',
            'role_id' => 'required|numeric|exists:roles,id',
            'username' => 'required|alpha_dash|max:191|unique:users,username',
            'email' => 'required|email|max:191|unique:users,email',
            'phone' => 'required|numeric|unique:users,phone',
            'password' => 'required|min:6|max:191|confirmed'
        ]);
        $params = $request->only(['first_name','last_name', 'role_id','username', 'email', 'phone', 'avatar', 'password']);
        $params['password'] = Hash::make($params['password']);
        if ($request->email_verification) {
            $params['email_verified_at'] = now();
        } else {
            $params['email_verified_at'] = null;
        }
        if ($request->phone_verification) {
            $params['phone_verified_at'] = now();
        } else {
            $params['phone_verified_at'] = null;
        }
        if ($request->push_notification) {
            $params['push_notification'] = 1;
        } else {
            $params['push_notification'] = 0;
        }
        if ($request->status) {
            $params['status'] = 1;
        } else {
            $params['status'] = 0;
        }
        if ($request->enabled_2fa) {
            $params['enabled_2fa'] = 1;
        } else {
            $params['enabled_2fa'] = 0;
        }
        if ($request->verified_2fa) {
            $params['verified_2fa'] = 1;
        } else {
            $params['verified_2fa'] = 0;
        }
        try {
            DB::beginTransaction();
            $user = User::create($params);
            if (!$user) throw new \Exception('Unable to create new user');
            event(new UserRegistered($user));
            DB::commit();
            return redirect()->route('staff.user.index')->withSuccess('New user created successfully');
        } catch (\Exception $exception) {
            DB::rollBack();
            return redirect()->back()->withInput()->withErrors($exception->getMessage());
        }
    }

    /**
     * Display the specified resource.
     *
     * @param \App\Models\User $user
     * @return \Illuminate\Http\Response
     */
    public function show(User $user)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param \App\Models\User $user
     * @return \Illuminate\Http\Response
     */
    public function edit(User $user)
    {
        if ($user == null) return abort(404);
        $roles = Role::where('for','user')->get();
        return view('staff.user.edit', compact('user', 'roles'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @param \App\Models\User $user
     * @return \Illuminate\Http\Response|void
     */
    public function update(Request $request, User $user)
    {
        if ($user == null) return abort(404);
        $request->validate([
            'first_name' => 'required|string|max:191',
            'last_name' => 'required|string|max:191',
            'role_id' => 'required|numeric|exists:roles,id',
            'username' => 'required|alpha_dash|max:191|unique:users,username,' . $user->id,
            'email' => 'required|email|max:191|unique:users,email,' . $user->id,
            'phone' => 'required|numeric|unique:users,phone,' . $user->id,
            'password' => 'nullable|min:6|max:191|confirmed'
        ]);

        $params = $request->only(['first_name','last_name', 'role_id','username', 'email', 'phone', 'avatar']);
        if ($request->password) $params['password'] = Hash::make($params['password']);
        if ($request->email_verification) {
            $params['email_verified_at'] = $user->email_verified_at ?? now();
        } else {
            $params['email_verified_at'] = null;
        }
        if ($request->phone_verification) {
            $params['phone_verified_at'] = $user->phone_verified_at ?? now();
        } else {
            $params['phone_verified_at'] = null;
        }
        if ($request->push_notification) {
            $params['push_notification'] = 1;
        } else {
            $params['push_notification'] = 0;
        }
        if ($request->status) {
            $params['status'] = 1;
        } else {
            $params['status'] = 0;
        }
        if ($request->enabled_2fa) {
            $params['enabled_2fa'] = 1;
        } else {
            $params['enabled_2fa'] = 0;
        }
        if ($request->verified_2fa) {
            $params['verified_2fa'] = 1;
        } else {
            $params['verified_2fa'] = 0;
        }
        try {
            $r = $user->update($params);
            if (!$r) throw new \Exception('Unable to update the user info');
            return redirect()->back()->withSuccess('User updated successfully');
        } catch (\Exception $exception) {
            return redirect()->back()->withInput()->withErrors($exception->getMessage());
        }
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param \App\Models\User $user
     * @return void
     */
    public function destroy(User $user)
    {
        if ($user == null) return abort(404);
        try {
            $r = $user->delete();
            if (!$r) throw new \Exception('Unable to delete user');
        } catch (\Exception $exception) {
            return redirect()->back()->withErrors($exception->getMessage());
        }
        return redirect()->back()->withSuccess('User deleted successfully');
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param \App\Models\User $user
     * @return \Illuminate\Http\Response
     */
    public function wallets(User $user)
    {
        if ($user == null) return abort(404);
        return view('staff.user.wallet', compact('user'));
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param \App\Models\Wallet $wallet
     * @return \Illuminate\Http\Response
     */
    public function updateWalletBalance(Request $request, Wallet $wallet)
    {
        if ($wallet == null) return abort(404);
        $request->validate([
            'action_type' => ['required', Rule::in(['add','deduct'])],
            'action_title' => 'required|string|max:191',
            'amount' => 'required|numeric|gt:0',
        ]);
        if (($request->action_type == 'deduct') && ($wallet->balance<$request->amount)) {
            throw ValidationException::withMessages(['amount' => 'Requested amount should less than or equal to the balance.']);
        }
        try {
            DB::beginTransaction();
            $type = '+';
            if ($request->action_type == 'add'){
                $wallet->balance+=$request->amount;
                $wallet->save();
            }
            elseif ($request->action_type == 'deduct'){
                $wallet->balance-=$request->amount;
                $wallet->save();
                $type= '-';
            }
            $transaction = $wallet->user->transactions()->create([
                'track' => Transaction::generateTrack(),
                'wallet_id' => $wallet->id,
                'title' => $request->action_title,
                'ref_type' => get_class($wallet),
                'ref_id' => $wallet->id,
                'type' => $type,
                'amount' => $request->amount,
                'matter' => 'wallet_balance_modification',
                'creator_type' => get_class(Auth::user()),
                'creator_id' => Auth::id(),
            ]);
            if (!$transaction) throw new \Exception('Unable to create transaction');
            event(new TransactionAdded($transaction));
            $staffs = Staff::all();
            Notification::send($staffs, new NotifyStaff('Wallet balance updated','$request->action_title', '/staff/user-wallet'));
            DB::commit();
            return redirect()->back()->withSuccess('Wallet balance updated successfully');
        } catch (\Exception $exception){
            DB::rollBack();
            throw ValidationException::withMessages(['amount' => $exception->getMessage()]);
        }
    }
}
