<?php

namespace App\Http\Controllers\Staff;

use App\Http\Controllers\Controller;
use App\Models\Input;
use App\Models\Role;
use App\Models\User;
use Illuminate\Http\Request;
use Illuminate\Support\Arr;
use Illuminate\Support\Facades\DB;
use Illuminate\Validation\ValidationException;
use Yajra\DataTables\DataTableAbstract;

class UserRoleController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {
        if ($request->ajax()) {
            $query = Role::query()->where('for','user');
            /* @var DataTableAbstract $table */
            $table = datatables()->of($query);
            $table->addColumn('caps', function (Role $role) {
                $allCaps = '';
                foreach ($role->caps as $key){
                    $allCaps=$allCaps .config('caps.user.' . $key) .' , ';
                }
                return $allCaps;
            });
            $table->addColumn('action', function (Role $role) {
                return '
                                                    <a href="' . route('staff.user-role.edit', $role->id) . '" class="btn btn-warning btn-edit nimmu-btn nimmu-btn-warning d-inline btn-xs"><i
                                                    class="fa fa-edit"></i></a>
                                            <button class="btn btn-danger btn-delete nimmu-btn nimmu-btn-danger btn-xs" data-id="' . $role->id . '"><i
                                                    class="fa fa-trash"></i></button>

                                                    ';
            });
            $table->rawColumns(['caps','action']);
            return $table->make(true);
        }
        return view('staff.roles.user.index');
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        return view('staff.roles.user.create');
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $request->validate([
            'name' => 'required|string|max:255',
            'caps' => 'required',
            'caps.*' => 'required',
            'fields' => 'nullable|array',
            'fields.*' => 'nullable|array',
            'fields.*.title' => 'required|string|max:191',
            'fields.*.input_type' => 'required',
            'fields.*.placeholder' => 'nullable|string|max:191',
            'fields.*.is_required' => 'nullable',
            'fields.*.sort' => 'nullable|numeric',
            'is_default' => 'nullable|in:0,1',
            'status' => 'nullable|in:0,1'
        ]);
        $old = Role::where([['for','user'],['status',1]])->count();
        if (!$old) {
            if ($request->is_default != 1) {
                throw ValidationException::withMessages(['is_default' => 'You must have to set your first role as default']);
            }
        }
        $roleInfo = $request->only(['name', 'caps', 'is_default', 'status']);
        $roleInfo['for'] = 'user';
        if ($request->is_default != 1) {
            $roleInfo['is_default'] = 0;
        }
        if ($request->status != 1) {
            $roleInfo['status'] = 0;
        }
        $inputs = [];
        if (is_array($request->fields)) {
            $inputs = array_map(function ($input) {
                $params = Arr::only($input, ['title', 'input_type', 'placeholder', 'is_required', 'sort']);
                if (isset($params['input_type'])) {
                    $params['type'] = $params['input_type'];
                }
                if (!isset($params['sort'])) {
                    $params['sort'] = 0;
                }
                if (isset($input['options']) && is_array($input['options'])) {
                    $params['options'] = array_map(function ($option) {
                        $p = Arr::only($option, ['title', 'sort']);
                        if (!isset($p['sort'])) $p['sort'] = 0;
                        return $p;
                    }, $input['options']);
                } else {
                    $params['options'] = [];
                }
                return $params;
            }, $request->fields);
        }
        try {
            DB::beginTransaction();
            if ($request->is_default == 1) {
                DB::table('roles')->where('for','user')->update(['is_default' => 0]);
            }
            $role = Role::create($roleInfo);
            if (!$role) return redirect()->back()->withErrors('Unable to create role');
            foreach ($inputs as $input) {
                /* @var Input $in */
                $in = $role->inputs()->create(Arr::except($input, ['options']));
                if (!$in) return redirect()->back()->withErrors('Unable to add input');
                if (is_array($input['options'])) {
                    $in->options()->createMany($input['options']);
                }
            }
            DB::commit();
            return redirect()->action('Staff\UserRoleController@index')->withSuccess('User Role created successfully');
        } catch (\Exception $exception) {
            DB::rollBack();
            return redirect()->back()->withErrors($exception->getMessage());
        }
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit(Role $role)
    {
        if (!$role) return abort(404);
        $role->load(['inputs' => function($q) {
            $q->with('options');
        }]);
        return view('staff.roles.user.edit', compact('role'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, Role $role)
    {
        if (!$role) return abort(404);
        $request->validate([
            'name' => 'required|string|max:199',
            'caps' => 'required',
            'caps.*' => 'required',
            'fields' => 'nullable|array',
            'fields.*' => 'nullable|array',
            'fields.*.title' => 'required|string|max:191',
            'fields.*.input_type' => 'required',
            'fields.*.placeholder' => 'nullable|string|max:191',
            'fields.*.is_required' => 'nullable',
            'fields.*.sort' => 'nullable|numeric',
            'is_default' => 'nullable|in:0,1',
            'status' => 'nullable|in:0,1'
        ]);

        $default = Role::where([['for','user'],['is_default',1],['status',1]])->first();
        if (is_null($default) || ($default->id == $role->id)) {
            if ($request->is_default != 1) {
                throw ValidationException::withMessages(['is_default' => 'You must have to set this role as default']);
            }
        }

        $inputs = [];
        if (is_array($request->fields)) {
            $inputs = array_map(function ($input) {
                $params = Arr::only($input, ['title', 'input_type', 'placeholder', 'is_required', 'sort']);
                if (isset($params['input_type'])) {
                    $params['type'] = $params['input_type'];
                }
                if (!isset($params['sort'])) {
                    $params['sort'] = 0;
                }
                if (isset($input['options']) && is_array($input['options'])) {
                    $params['options'] = array_map(function ($option) {
                        $p = Arr::only($option, ['title', 'sort']);
                        if (!isset($p['sort'])) $p['sort'] = 0;
                        return $p;
                    }, $input['options']);
                } else {
                    $params['options'] = [];
                }
                return $params;
            }, $request->fields);
        }
        try {
            DB::beginTransaction();
            if (($request->is_default == 1) && (!is_null($default) && ($default->id != $role->id))) {
                DB::table('roles')->where('for','user')->update(['is_default' => 0]);
            }
            $params = $request->only(['name', 'caps', 'is_default', 'status']);
            if ($request->is_default != 1) {
                $params['is_default'] = 0;
            }
            if ($request->status != 1) {
                $params['status'] = 0;
            }
            $res = $role->update($params);
            if (!$res) return redirect()->back()->withErrors('Unable to update role');
            $role->inputs()->delete();
            foreach ($inputs as $input) {
                /* @var Input $in */
                $in = $role->inputs()->create(Arr::except($input, ['options']));
                if (!$in) throw new \Exception(__('Unable to add input'));
                if (is_array($input['options'])) {
                    $in->options()->createMany($input['options']);
                }
            }
            DB::commit();
            return redirect()->back()->withSuccess('User Role updated successfully');
        } catch (\Exception $exception) {
            DB::rollBack();
            return redirect()->back()->withErrors($exception->getMessage());
        }
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy(Role $role)
    {
        if (!$role) return abort(404);
        try {
            DB::beginTransaction();
            $assignUsers = User::where('role_id',$role->id)->count();
            if ($assignUsers>0){
                return redirect()->back()->withErrors("You can't delete this role, it's already been used");
            }
            $role->inputs()->delete();
            $res = $role->delete();
            if (!$res) return redirect()->back()->withErrors('Unable to delete role. Maybe one or more staff exists in this role');
            DB::commit();
            return redirect()->back()->withSuccess('User Role deleted successfully');
        } catch (\Exception $exception) {
            DB::rollBack();
            return redirect()->back()->withErrors($exception->getMessage());
        }
    }
}
