<?php

namespace App\Http\Controllers\Staff;

use App\Models\Currency;
use App\Events\TransactionAdded;
use App\Events\WithdrawAccepted;
use App\Http\Controllers\Controller;
use App\Models\User;
use App\Models\Transaction;
use App\Models\Wallet;
use App\Models\Withdraw;
use Carbon\Carbon;
use Illuminate\Auth\Access\AuthorizationException;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use Yajra\DataTables\DataTableAbstract;

class WithdrawController extends Controller
{
    public function index(Request $request)
    {
        if ($request->ajax()) {
            $query = Withdraw::query()->with(['user', 'method','inputValues.option']);
            if (is_numeric($request->status) && in_array($request->status, [0, 1, 2])) {
                $query->where('status', $request->status);
            }
            if ($request->start_date && ($start_date = Carbon::createFromFormat('d-m-Y', $request->start_date))) {
                $query->whereDate('created_at', '>=', $start_date);
            }
            if ($request->end_date && ($end_date = Carbon::createFromFormat('d-m-Y', $request->end_date))) {
                $query->whereDate('created_at', '<=', $end_date);
            }
            /* @var DataTableAbstract $table */
            $table = datatables()->of($query);
            $table->editColumn('created_at', function (Withdraw $withdraw) {
                return $withdraw->created_at->format('M d, Y h:i a');
            });
            $table->editColumn('amount', function (Withdraw $withdraw) {
                $currency = $withdraw->wallet->currency;
                return sprintf('%s %s', $withdraw->amount, $currency->code);
            });
            $table->editColumn('charge', function (Withdraw $withdraw) {
                $currency = $withdraw->wallet->currency;
                return sprintf('%s %s', $withdraw->charge, $currency->code);
            });
            $table->editColumn('user.username', function (Withdraw $withdraw) {
                if ($withdraw->user instanceof User) return ('<a href="' . route('staff.user.edit', $withdraw->user->id) . '">' .$withdraw->user->username . '</a>');
            });
            $table->editColumn('user.username', function (Withdraw $withdraw) {
                return ('<div class="d-flex">
                            <div class="image mr-2">
                                <img class="img-thumbnail" style="min-width: 50px;height: 50px;"
                                     src="' . $withdraw->user->avatar . '"
                                     alt="' . $withdraw->user->username . '">
                            </div>
                            <div class="meta">
                                <p>
                                    <a href="' . route('staff.user.edit', $withdraw->user->id) . '">' . $withdraw->user->first_name .' ' .$withdraw->user->last_name . ' (' . $withdraw->user->username .')'.'</a>
                                </p>
                            </div>
                        </div>');
            });
            $table->editColumn('status', function (Withdraw $withdraw) {
                if ($withdraw->status == 0) return '<span class="badge badge-warning">Pending</span>';
                if ($withdraw->status == 1) return '<span class="badge badge-success">Accepted</span>';
                if ($withdraw->status == 2) return '<span class="badge badge-danger">Rejected</span>';
            });
            $table->addColumn('action', function (Withdraw $withdraw) {
                $actions = sprintf('<button class="btn btn-outline-info btn-sm nimmu-btn nimmu-btn-outline-info btn-show" data-fields=\'%s\'><i class="fa fa-eye"></i></button>', json_encode($withdraw->inputValues));
                if ($withdraw->status == 0) {
                    $actions .= sprintf('<button class="btn btn-primary btn-sm nimmu-btn nimmu-btn-primary btn-approve ml-1" data-id="%s"><i class="fa fa-check-circle"></i></button>', $withdraw->id);
                    $actions .= sprintf('<button class="btn btn-outline-danger btn-sm nimmu-btn nimmu-btn-outline-danger btn-refund ml-1" data-id="%s"><i class="fa fa-times"></i></button>', $withdraw->id);
                }
                return $actions;
            });
            $table->rawColumns(['user.username','status', 'action']);
            return $table->make(true);
        }
        return view('staff.withdraw.index');
    }

    public function update(Request $request, Withdraw $withdraw)
    {
        if (!$withdraw) return abort(404);
        if ($withdraw->status != 0) throw new AuthorizationException;
        $request->validate([
            'action' => 'required|in:accept,reject'
        ]);
        if ($request->action == 'accept') {
            try {
                DB::beginTransaction();
                $r = $withdraw->update([
                    'status' => 1
                ]);
                if (!$r) throw new \Exception('Unable to update withdraw request');
                DB::commit();
                event(new WithdrawAccepted($withdraw));
                return redirect()->back()->withSuccess('Withdraw request accepted successfully');
            } catch (\Exception $exception) {
                DB::rollBack();
                return redirect()->back()->withErrors($exception->getMessage());
            }
        } else {
            $user = $withdraw->user;
            $userWallet = $withdraw->wallet;
            try {
                DB::beginTransaction();
                $r = $withdraw->update([
                    'status' => 2
                ]);
                if (!$r) throw new \Exception('Unable to update withdraw request');
                $r2 = $userWallet->update([
                    'balance' => ($userWallet->balance + $withdraw->amount + $withdraw->charge)
                ]);
                if (!$r2) throw new \Exception('Unable to update user balance');
                /* @var Transaction $transaction */
                $transaction = $user->transactions()->create([
                    'track' => Transaction::generateTrack(),
                    'wallet_id' => $withdraw->wallet_id,
                    'title' => 'Withdraw Refund',
                    'user_type' => get_class($user),
                    'user_id' => $user->id,
                    'ref_type' => get_class($withdraw),
                    'ref_id' => $withdraw->id,
                    'type' => '+',
                    'amount' => ($withdraw->amount + $withdraw->charge),
                    'matter' => 'withdraw_refund',
                    'creator_type' => get_class(Auth::guard('staff')->user()),
                    'creator_id' => Auth::guard('staff')->id(),
                ]);
                if (!$transaction) throw new \Exception('Unable to create transaction');
                DB::commit();
                event(new TransactionAdded($transaction));
                return redirect()->back()->withSuccess('Withdraw request refunded successfully');
            } catch (\Exception $exception) {
                DB::rollBack();
                return redirect()->back()->withErrors($exception->getMessage());
            }
        }
    }
}
