<?php

namespace App\Http\Controllers\Staff;

use App\Models\Input;
use App\Models\Currency;
use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Support\Arr;
use Illuminate\Support\Facades\DB;
use Illuminate\Validation\ValidationException;
use Yajra\DataTables\DataTableAbstract;
use App\Models\WithdrawMethod;

class WithdrawMethodController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {
        $currency = Currency::getDefaultCurrency();
        $query = WithdrawMethod::query();
        if ($request->ajax()) {
            /* @var DataTableAbstract $table */
            $table = datatables()->of($query);
            $table->editColumn('min', function (WithdrawMethod $withdrawMethod) use ($currency) {
                return $withdrawMethod->min == -1 ? '<span class="badge badge-info">no limit</span>' : '<span class="badge badge-info">' . $withdrawMethod->min . ' ' . $currency->code . '</span>';
            });
            $table->editColumn('max', function (WithdrawMethod $withdrawMethod) use ($currency) {
                return $withdrawMethod->max == -1 ? '<span class="badge badge-info">no limit</span>' : '<span class="badge badge-info">' . $withdrawMethod->max . ' ' . $currency->code . '</span>';
            });
            $table->addColumn('charge', function (WithdrawMethod $withdrawMethod) use ($currency) {
                return sprintf('<span class="badge badge-info">%s%% + %s %s</span>', $withdrawMethod->percent_charge, $withdrawMethod->fixed_charge, $currency->code);
            });
            $table->editColumn('status', function (WithdrawMethod $withdrawMethod) use ($currency) {
                return $withdrawMethod->status == 1 ? '<span class="badge badge-success">Enabled</span>' : '<span class="badge badge-danger">Disabled</span>';
            });
            $table->addColumn('actions', function (WithdrawMethod $withdrawMethod) {
                return '<a href="' . route('staff.setting.withdraw-method.edit', $withdrawMethod->id) . '" class="btn btn-warning btn-edit nimmu-btn nimmu-btn-warning"><i class="fa fa-edit"></i></a>';
            });
            $table->rawColumns(['min', 'max', 'charge', 'status', 'actions']);
            return $table->make(true);
        }
        return view('staff.setting.withdraw-method.index', compact('currency'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $all_currencies = Currency::where('status',1)->get();
        return view('staff.setting.withdraw-method.create', compact('all_currencies'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $request->validate([
            'name' => 'required|string|max:191',
            'min' => 'nullable|numeric',
            'max' => 'nullable|numeric',
            'percent_charge' => 'nullable|numeric',
            'fixed_charge' => 'nullable|numeric',
            'supported_currencies' => 'required|array',
            'supported_currencies.*' => 'required',
            'fields' => 'nullable|array',
            'fields.*' => 'nullable|array',
            'fields.*.title' => 'required|string|max:191',
            'fields.*.input_type' => 'required',
            'fields.*.placeholder' => 'nullable|string|max:191',
            'fields.*.is_required' => 'nullable',
            'fields.*.sort' => 'nullable|numeric',
        ]);
        $params = $request->only(['name', 'description', 'min', 'max', 'percent_charge', 'fixed_charge']);
        $params['status'] = $request->status?1:0;
        if (!$request->min) {
            $params['min'] = -1;
        }
        if (!$request->max) {
            $params['max'] = -1;
        }
        if (!$request->percent_charge) {
            $params['percent_charge'] = 0;
        }
        if (!$request->fixed_charge) {
            $params['fixed_charge'] = 0;
        }
        //        For Input Options
        $inputs = [];
        if (is_array($request->fields)) {
            $inputs = array_map(function ($input) {
                $params = Arr::only($input, ['title', 'input_type', 'placeholder', 'is_required', 'sort']);
                if (isset($params['input_type'])) {
                    $params['type'] = $params['input_type'];
                }
                if (!isset($params['is_required'])) {
                    $params['is_required'] = 0;
                }
                if (!isset($params['sort'])) {
                    $params['sort'] = 0;
                }
                if (isset($input['options']) && is_array($input['options'])) {
                    $params['options'] = array_map(function ($option) {
                        $p = Arr::only($option, ['title', 'sort']);
                        if (!isset($p['sort'])) $p['sort'] = 0;
                        return $p;
                    }, $input['options']);
                } else {
                    $params['options'] = [];
                }
                return $params;
            }, $request->fields);
        }
        //        ================

        try {
            DB::beginTransaction();
            $withdrawMethod = WithdrawMethod::create($params);
            $withdrawMethod->currencies()->attach($request->supported_currencies);
            if (!$withdrawMethod) throw new \Exception('Unable to save method');
            foreach ($inputs as $input) {
                /* @var Input $in */
                $in = $withdrawMethod->inputs()->create(Arr::except($input, ['options']));
                if (!$in) throw new \Exception(__('Unable to add input'));
                if (is_array($input['options'])) {
                    $in->options()->createMany($input['options']);
                }
            }
            DB::commit();
            return redirect()->route('staff.setting.withdraw-method.index', $withdrawMethod->id)->withSuccess('Withdraw method created successfully');
        } catch (\Exception $exception) {
            DB::rollBack();
            throw ValidationException::withMessages(['name' => $exception->getMessage()]);
        }
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit(WithdrawMethod $withdrawMethod)
    {
        if (!$withdrawMethod) return abort(404);
        $withdrawMethod->load(['inputs' => function($q) {
            $q->with('options');
        }]);
        $all_currencies = Currency::where('status',1)->get();
        return view('staff.setting.withdraw-method.edit', compact('all_currencies', 'withdrawMethod'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, WithdrawMethod $withdrawMethod)
    {
        if (!$withdrawMethod) return abort(404);
        $request->validate([
            'name' => 'required|string|max:191',
            'min' => 'nullable|numeric',
            'max' => 'nullable|numeric',
            'percent_charge' => 'nullable|numeric',
            'fixed_charge' => 'nullable|numeric',
            'supported_currencies' => 'required|array',
            'supported_currencies.*' => 'required',
            'fields' => 'nullable|array',
            'fields.*' => 'nullable|array',
            'fields.*.title' => 'required|string|max:191',
            'fields.*.input_type' => 'required',
            'fields.*.placeholder' => 'nullable|string|max:191',
            'fields.*.is_required' => 'nullable',
            'fields.*.sort' => 'nullable|numeric'
        ]);
        $params = $request->only(['name', 'description', 'min', 'max', 'percent_charge', 'fixed_charge']);
        $params['status'] = $request->status?1:0;
        if (!$request->min) {
            $params['min'] = -1;
        }
        if (!$request->max) {
            $params['max'] = -1;
        }
        if (!$request->percent_charge) {
            $params['percent_charge'] = 0;
        }
        if (!$request->fixed_charge) {
            $params['fixed_charge'] = 0;
        }
        //        For Input Options
        $inputs = [];
        if (is_array($request->fields)) {
            $inputs = array_map(function ($input) {
                $params = Arr::only($input, ['title', 'input_type', 'placeholder', 'is_required', 'sort']);
                if (isset($params['input_type'])) {
                    $params['type'] = $params['input_type'];
                }
                if (!isset($params['is_required'])) {
                    $params['is_required'] = 0;
                }
                if (!isset($params['sort'])) {
                    $params['sort'] = 0;
                }
                if (isset($input['options']) && is_array($input['options'])) {
                    $params['options'] = array_map(function ($option) {
                        $p = Arr::only($option, ['title', 'sort']);
                        if (!isset($p['sort'])) $p['sort'] = 0;
                        return $p;
                    }, $input['options']);
                } else {
                    $params['options'] = [];
                }
                return $params;
            }, $request->fields);
        }
        //        ==================


        try {
            DB::beginTransaction();
            $r = $withdrawMethod->update($params);
            if (!$r) throw new \Exception('Unable to update method');
            $withdrawMethod->currencies()->sync($request->supported_currencies);
            $withdrawMethod->inputs()->delete();
            foreach ($inputs as $input) {
                /* @var Input $in */
                $in = $withdrawMethod->inputs()->create(Arr::except($input, ['options']));
                if (!$in) throw new \Exception(__('Unable to add input'));
                if (is_array($input['options'])) {
                    $in->options()->createMany($input['options']);
                }
            }
            DB::commit();
            return redirect()->back()->withSuccess('Withdraw method updated successfully');
        } catch (\Exception $exception) {
            DB::rollBack();
            throw ValidationException::withMessages(['name' => $exception->getMessage()]);
        }
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        //
    }
}
