<?php

namespace App\Http\Controllers\Staff;

use App\Http\Controllers\Controller;
use App\Models\Language;
use App\Models\WorkFlow;
use Illuminate\Database\Eloquent\Collection;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;

class WorkFlowController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $workFlows = WorkFlow::all();
        return view('staff.setting.workflow.index',compact('workFlows'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $languages = Language::query()->where('status', 1)->get();
        return view('staff.setting.workflow.create',compact('languages'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $request->validate([
            'title' => 'required|string|max:255',
            'details' => 'required|string'
        ]);
        DB::beginTransaction();
        try {
            $params = $request->only(['title', 'details']);
            /* @var WorkFlow|null $workFlow */

            $workFlow = WorkFlow::create($params);

            if (is_array($request->lang)) {
                $translations = [];
                foreach ($request->lang as $code => $columns) {
                    foreach ($columns as $column => $content) {
                        $translations[] = [
                            'language' => $code,
                            'column_name' => $column,
                            'content' => $content
                        ];
                    }
                }
                $workFlow->translations()->createMany($translations);
            }
        } catch (\Exception $exception) {
            DB::rollBack();
            return redirect()->back()->withInput()->withErrors($exception->getMessage());
        }
        DB::commit();
        return redirect()->route('staff.setting.workflow.index')->withSuccess('New Step added successfully');
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($workFlowID)
    {
        $workFlow = WorkFlow::findOrFail($workFlowID);

        /* @var Collection $translations */
        $translations = $workFlow->translations;
        $translations = $translations->groupBy('language')->map(function ($items) {
            return collect($items)->pluck('content', 'column_name');
        })->toArray();
        $languages = Language::query()->where('status', 1)->get();
        return view('staff.setting.workflow.edit', compact( 'workFlow', 'translations', 'languages'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $workFlowID)
    {
        $workFlow = WorkFlow::findOrFail($workFlowID);
        $request->validate([
            'title' => 'required|string|max:255',
            'details' => 'required|string'
        ]);
        DB::beginTransaction();
        try {
            $params = $request->only(['title', 'details']);

            $workFlow->update($params);

            $workFlow->translations()->delete();
            if (is_array($request->lang)) {
                $translations = [];
                foreach ($request->lang as $code => $columns) {
                    foreach ($columns as $column => $content) {
                        $translations[] = [
                            'language' => $code,
                            'column_name' => $column,
                            'content' => $content
                        ];
                    }
                }
                $workFlow->translations()->createMany($translations);
            }
        } catch (\Exception $exception) {
            DB::rollBack();
            return redirect()->back()->withInput()->withErrors($exception->getMessage());
        }
        DB::commit();
        return redirect()->route('staff.setting.workflow.index')->withSuccess('Workflow step updated successfully');
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($workFlowID)
    {
        $workFlow = WorkFlow::findOrFail($workFlowID);
        DB::beginTransaction();
        try {
            $workFlow->translations()->delete();
            $r = $workFlow->delete();
            if (!$r) throw new \Exception('Unable to delete workflow step');
        } catch (\Exception $exception) {
            DB::rollBack();
            return redirect()->back()->withInput()->withErrors($exception->getMessage());
        }
        DB::commit();
        return redirect()->route('staff.setting.workflow.index')->withSuccess('Workflow step deleted successfully');
    }
}
