<?php

namespace App\Http\Controllers\User;

use App\Events\PaymentRequestAccepted;
use App\Events\TransactionAdded;
use App\Http\Controllers\Controller;
use App\Models\PaymentRequest;
use App\Models\Transaction;
use App\Models\User;
use App\Models\Wallet;
use Illuminate\Auth\Access\AuthorizationException;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use Illuminate\Validation\ValidationException;

class PaymentRequestController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $payment_requests = PaymentRequest::where('to_id',Auth::id())->get();
        return view('user.payment_requests.index',compact('payment_requests'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $request->validate([
            'to_id' => 'required|exists:users,id',
            'currency_id' => 'required|exists:currencies,id',
            'amount' => 'required|numeric|min:0',
            'note' => 'required|string|max:250',
        ]);

        $params = $request->all();
        $params['by_id']=Auth::id();
        $paymentRequest = PaymentRequest::create($params);
        if (!$paymentRequest) throw new \Exception('Unable to create payment request');
        return redirect()->route('dashboard.wallet')->withSuccess('Payment Request Send Successfully');
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, PaymentRequest $paymentRequest)
    {
        if (!$paymentRequest) return abort(404);
        if ($paymentRequest->status != 'pending') throw new AuthorizationException;
        $request->validate([
            'action' => 'required|in:approved,rejected'
        ]);
        if ($request->action == 'approved') {
            $user = Auth::user();
            $requester = User::findOrFail($paymentRequest->by_id);
            $userWallet = Wallet::where([['user_id',$user->id],['currency_id',$paymentRequest->currency_id]])->firstOrFail();
            $requesterWallet = Wallet::where([['user_id',$paymentRequest->by_id],['currency_id',$paymentRequest->currency_id]])->firstOrFail();
            if ($userWallet->balance<$paymentRequest->amount) throw ValidationException::withMessages(['error' => "You don't have enough balance"]);
            try {
                DB::beginTransaction();

                $r = $paymentRequest->update([
                    'status' => 'approved'
                ]);
                if (!$r) throw new \Exception('Unable to update payment request');
                event(new PaymentRequestAccepted($paymentRequest));

                $r1 = $requesterWallet->update([
                    'balance' => ($requesterWallet->balance + $paymentRequest->amount)
                ]);

                $r2 = $userWallet->update([
                    'balance' => ($userWallet->balance - $paymentRequest->amount)
                ]);
                if (!$r1 || !$r2) throw new \Exception('Unable to update user balance');
                /* @var Transaction $transaction */
                $userTransaction = $user->transactions()->create([
                    'track' => Transaction::generateTrack(),
                    'wallet_id' => $userWallet->id,
                    'title' => 'Payment Request Accepted',
                    'ref_type' => get_class($userWallet),
                    'ref_id' => $userWallet->id,
                    'type' => '-',
                    'amount' => $paymentRequest->amount,
                    'matter' => 'payment_request_accepted',
                    'creator_type' => get_class($user),
                    'creator_id' => $user->id,
                ]);

                $transaction = $requester->transactions()->create([
                    'track' => Transaction::generateTrack(),
                    'wallet_id' => $requesterWallet->id,
                    'title' => 'Payment Request Accepted',
                    'ref_type' => get_class($requesterWallet),
                    'ref_id' => $requesterWallet->id,
                    'type' => '+',
                    'amount' => $paymentRequest->amount,
                    'matter' => 'payment_request_accepted',
                    'creator_type' => get_class($user),
                    'creator_id' => $user->id,
                ]);
                if (!$userTransaction || !$transaction) throw new \Exception('Unable to create transaction');
                event(new TransactionAdded($transaction));
                DB::commit();

                return redirect()->back()->withSuccess('Payment request accepted successfully');
            } catch (\Exception $exception) {
                DB::rollBack();
                return redirect()->back()->withErrors($exception->getMessage());
            }
        } else {
            try {
                DB::beginTransaction();
                $r = $paymentRequest->update([
                    'status' => 'rejected'
                ]);
                if (!$r) throw new \Exception('Unable to update payment request');
                DB::commit();
                return redirect()->back()->withSuccess('Withdraw request refunded successfully');
            } catch (\Exception $exception) {
                DB::rollBack();
                return redirect()->back()->withErrors($exception->getMessage());
            }
        }
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        //
    }
}
