<?php

namespace App\Http\Controllers\User;

use App\Http\Controllers\Controller;
use App\Models\PaymentGateway;
use App\Models\Store;
use App\Models\Transaction;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use Illuminate\Validation\ValidationException;

class StoreController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $stores = Store::where('user_id',Auth::id())->get();
        return view('user.store.index',compact('stores'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $paymentGateways = PaymentGateway::where('status',1)->get();
        return view('user.store.create',compact('paymentGateways'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $request->validate([
            'name' => 'required|string|max:200',
            'supported_payment_gateway' => 'nullable|array'
        ]);
        $params = $request->all();
        $params['user_id'] = Auth::id();
        $params['api_key'] = Store::generateApiKey();
        $params['api_secret'] = Store::generateApiSecret();
        try {
            DB::beginTransaction();
            $store = Store::create($params);
            if (!$store) throw new \Exception('Unable to create new store');
            $store->payment_gateways()->attach($request->supported_payment_gateway);
            DB::commit();
            return redirect()->route('stores.index')->withSuccess('New store created successfully');
        } catch (\Exception $exception) {
            DB::rollBack();
            throw ValidationException::withMessages(['name' => $exception->getMessage()]);
        }
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show(Store $store)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit(Store $store)
    {
        if (!$store || ($store->user_id != Auth::id())) return abort(404);
        $paymentGateways = PaymentGateway::where('status',1)->get();
        return view('user.store.edit',compact('store','paymentGateways'));
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function transactions(Store $store)
    {
        if (!$store || ($store->user_id != Auth::id())) return abort(404);
        $transactions=Transaction::where([['ref_type',get_class($store)],['ref_id',$store->id]])->paginate(10);
        return view('user.store.transaction',compact('transactions'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, Store $store)
    {
        if (!$store || ($store->user_id != Auth::id()))
        $request->validate([
            'name' => 'required|string|max:200',
            'supported_payment_gateway' => 'nullable|array'
        ]);

        try {
            DB::beginTransaction();
            $params = $request->only('name');
            $r = $store->update($params);
            if (!$r) throw new \Exception('Unable to update store info');
            $store->payment_gateways()->sync($request->supported_payment_gateway);
            DB::commit();
            return redirect()->back()->withSuccess('Store info updated successfully');
        } catch (\Exception $exception) {
            DB::rollBack();
            throw ValidationException::withMessages(['name' => $exception->getMessage()]);
        }
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy(Store $store)
    {
        if (!$store || ($store->user_id != Auth::id())) throw ValidationException::withMessages(['error' => "Invalid store information"]);
        $res = $store->delete();
        if (!$res) throw new \Exception('Unable to delete store info');
        return redirect()->back()->withSuccess('Store deleted successfully');
    }

    public function regenerateApiKey($id){
        $store = Store::where([['id',$id],['user_id',Auth::id()]])->firstOrFail();
        $res = $store->update([
            'api_key' => Store::generateApiKey()
        ]);
        if (!$res) throw new \Exception('Something went wrong, please try again later.');
        return redirect()->back()->withSuccess('Api key regenerated successfully');
    }

    public function regenerateApiSecret($id){
        $store = Store::where([['id',$id],['user_id',Auth::id()]])->firstOrFail();
        $res = $store->update([
            'api_secret' => Store::generateApiSecret()
        ]);
        if (!$res) throw new \Exception('Something went wrong, please try again later.');
        return redirect()->back()->withSuccess('Api Secret regenerated successfully');
    }
}
