<?php

namespace App\Http\Controllers\User;

use App\Models\Currency;
use App\Events\TransactionAdded;
use App\Events\WithdrawAccepted;
use App\Models\Transaction;
use App\Models\User;
use App\Models\Wallet;
use App\Models\Withdraw;
use App\Models\WithdrawMethod;
use Carbon\Carbon;
use Illuminate\Auth\Access\AuthorizationException;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use Illuminate\Validation\ValidationException;
use Yajra\DataTables\DataTableAbstract;

class WithdrawController
{
    /**
     * Display a listing of the resource.
     *
     * @param Request $request
     * @return \Illuminate\Contracts\Foundation\Application|\Illuminate\Contracts\View\Factory|\Illuminate\Http\JsonResponse|\Illuminate\View\View
     */
    public function index(Request $request)
    {
        $user = Auth::user();
        $withdrawals=Withdraw::where([['user_type',get_class($user)],['user_id',$user->id]])->paginate(10);
        return view('user.withdraw.index',compact('withdrawals'));
    }

    public function create(Request $request)
    {
        $wallets = Wallet::with('currency')->where('user_id',Auth::id())->get();
        $methods = WithdrawMethod::query()->where('status', 1)->get();
        $methods->load(['inputs' => function($q) {
            $q->with('options')->orderBY('sort');
        }]);
        return view('user.withdraw.create', compact('methods','wallets'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $request->validate([
            'wallet_id' => 'required|exists:wallets,id',
            'withdraw_method_id' => 'required|exists:withdraw_methods,id',
            'amount' => 'required|numeric|min:0'
        ]);
        /* @var User $user */
        $user = Auth::user();
        /* @var WithdrawMethod $withdrawMethod */
        $userWallet = Wallet::query()->where([['id', $request->wallet_id],['user_id',$user->id]])->firstOrFail();
        $withdrawMethod = WithdrawMethod::query()->where('id', $request->withdraw_method_id)->where('status', 1)->firstOrFail();
        if ($request->amount == 0) throw ValidationException::withMessages(['error' => 'Amount must be greater then 0']);
        if ($withdrawMethod->min != -1 && $request->amount < $withdrawMethod->min) throw ValidationException::withMessages(['error' => 'Amount is less then method minimum amount']);
        if ($withdrawMethod->max != -1 && $request->amount > $withdrawMethod->max) throw ValidationException::withMessages(['error' => 'Amount is greater then method maximum amount']);
        $charge = round(($withdrawMethod->fixed_charge + (($request->amount * $withdrawMethod->percent_charge) / 100)), 2);
        $gross = $request->amount+$charge;
        if ($gross > $userWallet->balance) throw ValidationException::withMessages(['amount' => 'Not enough balance']);
        //        For Input Options
        $inputValues = [];
        if (is_array($request->fields)) {
            $index=0;
            foreach ($request->fields as $field){
                if (isset($field['input_option_id'])){
                    if (is_array($field['input_option_id'])){
                        foreach ($field['input_option_id'] as $input_option){
                            $inputValues[$index]['input_id'] = $field['input_id'];
                            $inputValues[$index]['title'] = $field['title'];
                            $inputValues[$index]['input_option_id'] = $input_option;
                            $index++;
                        }
                    }
                    else{
                        $inputValues[$index]['input_id'] = $field['input_id'];
                        $inputValues[$index]['title'] = $field['title'];
                        $inputValues[$index]['input_option_id'] = $field['input_option_id'];
                        $index++;
                    }
                }
                else{
                    $inputValues[$index]['input_id'] = $field['input_id'];
                    $inputValues[$index]['title'] = $field['title'];
                    if (isset($field['content'])){
                        $inputValues[$index]['content'] = $field['content'];
                    }
                    $index++;
                }
            }
        }
        //        ================
        try {
            DB::beginTransaction();
            $r = $userWallet->update([
                'balance' => ($userWallet->balance-$gross)
            ]);
            if (!$r) throw new \Exception('Unable to update balance');
            /* @var Withdraw $withdraw */
            $withdraw = $user->withdraws()->create([
                'wallet_id' => $request->wallet_id,
                'withdraw_method_id' => $request->withdraw_method_id,
                'amount' => $request->amount,
                'charge' => $charge,
            ]);
            if (!$withdraw) throw new \Exception('Unable to create withdraw');
            /* @var Transaction $transaction */
            $transaction = $user->transactions()->create([
                'track' => Transaction::generateTrack(),
                'wallet_id' => $withdraw->id,
                'title' => 'Withdraw request',
                'user_type' => get_class($user),
                'user_id' => $user->id,
                'ref_type' => get_class($withdraw),
                'ref_id' => $withdraw->id,
                'type' => '-',
                'amount' => $gross,
                'matter' => 'withdraw_request',
                'creator_type' => get_class($user),
                'creator_id' => $user->id,
            ]);
            if (!$transaction) throw new \Exception('Unable to create transaction');


            foreach ($inputValues as $inputValue) {
                $in = $withdraw->inputValues()->create($inputValue);
                if (!$in) throw new \Exception(__('Unable to add input'));
            }

            event(new TransactionAdded($transaction));
            DB::commit();
            return redirect()->route('dashboard.wallet')->withSuccess('Withdraw request created successfully');
        } catch (\Exception $exception) {
            DB::rollBack();
            throw ValidationException::withMessages(['error' => $exception->getMessage()]);
        }
    }
}
