<?php

namespace App\PaymentMethods;

use App\Events\TransactionAdded;
use App\Models\Currency;
use App\Models\Deposit;
use App\Models\Payment;
use App\Models\PaymentGateway;
use App\Models\Transaction;
use App\Models\User;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Razorpay\Api\Api;
use Razorpay\Api\Order;
use Razorpay\Api\Utility;

class RazorPay extends PaymentMethod
{
    public $supported_currencies = ['USD'];
    public $credentials = ['api_key', 'api_secret'];

    public function process(PaymentGateway $dbMethod, $payment)
    {
        $currency = $payment->currency;
        $api = new Api($dbMethod->credentials['api_key'], $dbMethod->credentials['api_secret']);
        if ($payment instanceof Deposit) {
            $amount = $payment->amount + $payment->charge;
            $type = 'deposit';
            $desc = "Deposit #" . $payment->id;
        }
        if ($payment instanceof Payment) {
            $amount = $payment->amount;
            $type = 'store_payment';
            $desc = "Payment #" . $payment->id;
        }
        /* @var Order $order */
        $order = $api->order->create([
            'receipt' => $type . '_' . $payment->id,
            'amount' => ($amount * 100),
            'currency' => $currency->code
        ]);
        cache()->put($type . '_' . $payment->id . '_razorpay_order_id', $order['id']);
        return view('payment.razor-pay', compact('currency', 'dbMethod', 'payment', 'order', 'type', 'amount', 'desc'));
    }

    public function ipn(Request $request, \App\Models\PaymentGateway $dbMethod, $payment)
    {
        $currency = $payment->currency;
        if ($payment instanceof Deposit) {
            $type = 'deposit';
            $amount = $payment->amount + $payment->charge;
        }
        if ($payment instanceof Payment) {
            $type = 'store_payment';
            $amount = $payment->amount;
        }
        $razorpay_order_id_stored = cache()->get($type . '_' . $payment->id . '_razorpay_order_id');
        if (!$razorpay_order_id_stored) return redirect()->route('payment.failed', ['type' => $type, 'track' => encrypt($payment->id)]);

        $api = new Api($dbMethod->credentials['api_key'], $dbMethod->credentials['api_secret']);
        try {
            $api->utility->verifyPaymentSignature([
                'razorpay_signature' => $request->razorpay_signature,
                'razorpay_payment_id' => $request->razorpay_payment_id,
                'razorpay_order_id' => $razorpay_order_id_stored
            ]);
        } catch (\Exception $exception) {
            return redirect()->route('payment.failed', ['type' => $type, 'track' => encrypt($payment->id)])->withErrors($exception->getMessage());
        }

        try {
            $order = $api->order->fetch($razorpay_order_id_stored);
            if ($order['status'] != 'paid') throw new \Exception('Payment status not paid');
            if ($order['amount_paid'] < ($amount * 100)) throw new \Exception('Paid less then main amount');
            if (strtolower($order['currency']) != strtolower($currency->code)) throw new \Exception('Currency not matched');
        } catch (\Exception $exception) {
            return redirect()->route('payment.failed', ['type' => $type, 'track' => encrypt($payment->id)])->withErrors($exception->getMessage());
        }

        try {
            DB::beginTransaction();
            if ($payment instanceof Deposit && $payment->status < 1) {
                $payment->update([
                    'status' => 1
                ]);
                /* @var User $user */
                $user = $payment->user;
                /* @var \App\Models\Wallet $wallet */
                $wallet = $user->wallets()->firstOrCreate(['currency_id' => $payment->currency_id]);
                $wallet->update([
                    'balance' => ($payment->amount + $wallet->balance)
                ]);
                /* @var Transaction $transaction */
                $transaction = $user->transactions()->create([
                    'wallet_id' => $wallet->id,
                    'track' => Transaction::generateTrack(),
                    'title' => 'Deposited via ' . optional($payment->payment_gateway)->name,
                    'ref_type' => get_class($payment),
                    'ref_id' => $payment->id,
                    'type' => '+',
                    'amount' => $payment->amount,
                    'matter' => 'deposit'
                ]);
                event(new TransactionAdded($transaction));
            }
            if ($payment instanceof Payment && $payment->status == 'initialized') {
                $payment->update([
                    'status' => 'completed'
                ]);
                /* @var User $user */
                $user = $payment->user;
                /* @var \App\Models\Wallet $wallet */
                $wallet = $user->wallets()->firstOrCreate(['currency_id' => $payment->currency_id]);
                $wallet->update([
                    'balance' => ($payment->amount + $wallet->balance)
                ]);
                /* @var Transaction $transaction */
                $transaction = $user->transactions()->create([
                    'wallet_id' => $wallet->id,
                    'track' => Transaction::generateTrack(),
                    'title' => 'Store Payment Received',
                    'ref_type' => get_class($payment->store),
                    'ref_id' => $payment->store_id,
                    'type' => '+',
                    'amount' => $payment->amount,
                    'matter' => 'store_payment',
                    'creator_type' => get_class($payment),
                    'creator_id' => $payment->id
                ]);
                event(new TransactionAdded($transaction));
            }
            DB::commit();
            if ($payment instanceof Payment) {
                $payment->sendSuccessIPNAndRedirectToSuccess();
            }
        } catch (\Exception $exception) {
            DB::rollBack();
            return redirect()->route('payment.failed', ['type' => $type, 'track' => encrypt($payment->id)])->withErrors($exception->getMessage());
        }
        return redirect()->route('payment.success', ['type' => $type, 'track' => encrypt($payment->id)])->withSuccess('Payment successfully completed');
    }
}
