<?php

namespace App\PaymentMethods;

use App\Events\TransactionAdded;
use App\Models\Currency;
use App\Models\Deposit;
use App\Models\Payment;
use App\Models\Transaction;
use App\Models\User;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Stripe\Checkout\Session;
use Stripe\Exception\ApiErrorException;
use Stripe\PaymentIntent;
use \Stripe\Stripe as StripeLib;

class Stripe extends PaymentMethod
{
    public $supported_currencies = ['USD'];
    public $credentials = ['publishable_key', 'secret_key'];

    function process(\App\Models\PaymentGateway $dbMethod, $payment)
    {
        $currency = $payment->currency;
        if ($payment instanceof Deposit) {
            $gross = $payment->amount + $payment->charge;
            $line_items = [[
                'name' => 'Deposit #' . $payment->id,
                'quantity' => 1,
                'amount' => $gross * 100,
                'currency' => $currency->code
            ]];
            $ipn_url = route('payment.ipn', ['type' => 'deposit', 'track' => encrypt($payment->id), 'session_id' => '{CHECKOUT_SESSION_ID}']);
            $cancel_url = route('payment.failed', ['type' => 'deposit', 'track' => encrypt($payment->id)]);
            $cache_key = 'stripe_session_deposit_' . $payment->id;
        }
        if ($payment instanceof Payment) {
            $gross = $payment->amount;
            $line_items = [[
                'name' => 'Payment #' . $payment->id,
                'quantity' => 1,
                'amount' => $gross * 100,
                'currency' => $currency->code
            ]];
            $ipn_url = route('payment.ipn', ['type' => 'store_payment', 'track' => encrypt($payment->id), 'session_id' => '{CHECKOUT_SESSION_ID}']);
            $cancel_url = route('payment.failed', ['type' => 'store_payment', 'track' => encrypt($payment->id)]);
            $cache_key = 'stripe_session_store_payment_' . $payment->id;
        }
        $p = [
            'payment_method_types' => ['card'],
            'line_items' => $line_items,
            'mode' => 'payment',
            'success_url' => $ipn_url,
            'cancel_url' => $cancel_url,
        ];
        StripeLib::setApiKey($dbMethod->credentials['secret_key']);
        $session = Session::create($p);
        $session_id = $session->id;
        cache()->put($cache_key, $session_id);
        return view('payment.stripe', compact('dbMethod', 'payment', 'currency', 'session_id'));
    }

    public function ipn(Request $request, \App\Models\PaymentGateway $dbMethod, $payment)
    {
        if ($payment instanceof Deposit) {
            $cache_key = 'stripe_session_deposit_' . $payment->id;
            $type = 'deposit';
        }
        if ($payment instanceof Payment) {
            $cache_key = 'stripe_session_store_payment_' . $payment->id;
            $type = 'store_payment';
        }
        $track = encrypt($payment->id);
        $stripe_session_ = cache()->get($cache_key);
        if (!$stripe_session_) return redirect()->route('payment.failed', ['type' => $type, 'track' => $track]);
        StripeLib::setApiKey($dbMethod->credentials['secret_key']);
        try {
            $checkout_session = Session::retrieve($stripe_session_);
        } catch (\Exception $exception) {
            return redirect()->route('payment.failed', ['type' => $type, 'track' => $track])->withErrors($exception->getMessage());
        }
        if (!$checkout_session) return redirect()->route('payment.failed', ['type' => $type, 'track' => $track])->withErrors(__('Transaction has been failed!'));
        try {
            $intent = PaymentIntent::retrieve($checkout_session->payment_intent);
        } catch (ApiErrorException $e) {
            return redirect()->route('payment.failed', ['type' => $type, 'track' => $track])->withErrors($e->getMessage());
        }
        if (!$intent) return redirect()->route('payment.failed', ['type' => $type, 'track' => $track])->withErrors(__('Unable to fetch the transaction details!'));
        if ($intent->status != 'succeeded') return redirect()->route('payment.failed', ['type' => $type, 'track' => $track])->withErrors(__('Transaction has been failed!'));
        try {
            DB::beginTransaction();
            if ($payment instanceof Deposit && $payment->status < 1) {
                $payment->update([
                    'status' => 1
                ]);
                /* @var User $user */
                $user = $payment->user;
                /* @var \App\Models\Wallet $wallet */
                $wallet = $user->wallets()->firstOrCreate(['currency_id' => $payment->currency_id]);
                $wallet->update([
                    'balance' => ($payment->amount + $wallet->balance)
                ]);
                /* @var Transaction $transaction */
                $transaction = $user->transactions()->create([
                    'wallet_id' => $wallet->id,
                    'track' => Transaction::generateTrack(),
                    'title' => 'Deposited via ' . optional($payment->payment_gateway)->name,
                    'ref_type' => get_class($payment),
                    'ref_id' => $payment->id,
                    'type' => '+',
                    'amount' => $payment->amount,
                    'matter' => 'deposit'
                ]);
                event(new TransactionAdded($transaction));
            }
            if ($payment instanceof Payment && $payment->status == 'initialized') {
                $payment->update([
                    'status' => 'completed'
                ]);
                /* @var User $user */
                $user = $payment->user;
                /* @var \App\Models\Wallet $wallet */
                $wallet = $user->wallets()->firstOrCreate(['currency_id' => $payment->currency_id]);
                $wallet->update([
                    'balance' => ($payment->amount + $wallet->balance)
                ]);
                /* @var Transaction $transaction */
                $transaction = $user->transactions()->create([
                    'wallet_id' => $wallet->id,
                    'track' => Transaction::generateTrack(),
                    'title' => 'Store Payment Received',
                    'ref_type' => get_class($payment->store),
                    'ref_id' => $payment->store_id,
                    'type' => '+',
                    'amount' => $payment->amount,
                    'matter' => 'store_payment',
                    'creator_type' => get_class($payment),
                    'creator_id' => $payment->id
                ]);
                event(new TransactionAdded($transaction));
            }
            DB::commit();
            if ($payment instanceof Payment) {
                $payment->sendSuccessIPNAndRedirectToSuccess();
            }
        } catch (\Exception $exception) {
            DB::rollBack();
            return redirect()->route('payment.failed', ['type' => $type, 'track' => encrypt($payment->id)])->withErrors($exception->getMessage());
        }
        return redirect()->route('payment.success', ['type' => $type, 'track' => encrypt($payment->id)])->withSuccess('Payment successfully completed');
    }
}
